# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_route53 as boto_route53

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoRoute53TestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_route53
    """

    def setup_loader_modules(self):
        return {boto_route53: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure the Route53 record is present.
        """
        name = "test.example.com."
        value = "1.1.1.1"
        zone = "example.com."
        record_type = "A"

        ret = {"name": name, "result": False, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[{}, {}, {"value": ""}, False])
        mock_bool = MagicMock(return_value=False)
        with patch.dict(
            boto_route53.__salt__,
            {"boto_route53.get_record": mock, "boto_route53.add_record": mock_bool},
        ):
            with patch.dict(boto_route53.__opts__, {"test": False}):
                comt = "Failed to add {0} Route53 record.".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(
                    boto_route53.present(name, value, zone, record_type), ret
                )

            with patch.dict(boto_route53.__opts__, {"test": True}):
                comt = "Route53 record {0} set to be added.".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    boto_route53.present(name, value, zone, record_type), ret
                )

                comt = "Route53 record {0} set to be updated.".format(name)
                ret.update({"comment": comt})
                self.assertDictEqual(
                    boto_route53.present(name, value, zone, record_type), ret
                )

            ret.update({"comment": "", "result": True})
            self.assertDictEqual(
                boto_route53.present(name, value, zone, record_type), ret
            )

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the Route53 record is deleted.
        """
        name = "test.example.com."
        zone = "example.com."
        record_type = "A"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[False, True])
        with patch.dict(boto_route53.__salt__, {"boto_route53.get_record": mock}):
            comt = "{0} does not exist.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(boto_route53.absent(name, zone, record_type), ret)

            with patch.dict(boto_route53.__opts__, {"test": True}):
                comt = "Route53 record {0} set to be deleted.".format(name)
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(boto_route53.absent(name, zone, record_type), ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import datetime
import logging

from tests.support.helpers import slowTest
from tests.support.unit import skipIf
from tests.unit.utils.scheduler.base import SchedulerTestsBase

try:
    import dateutil.parser

    HAS_DATEUTIL_PARSER = True
except ImportError:
    HAS_DATEUTIL_PARSER = False

log = logging.getLogger(__name__)


@skipIf(
    HAS_DATEUTIL_PARSER is False, "The 'dateutil.parser' library is not available",
)
class SchedulerPostponeTest(SchedulerTestsBase):
    """
    Validate the pkg module
    """

    def setUp(self):
        super(SchedulerPostponeTest, self).setUp()
        self.schedule.opts["loop_interval"] = 1

    def tearDown(self):
        self.schedule.reset()
        super(SchedulerPostponeTest, self).tearDown()

    @slowTest
    def test_postpone(self):
        """
        verify that scheduled job is postponed until the specified time.
        """
        job = {
            "schedule": {"job1": {"function": "test.ping", "when": "11/29/2017 4pm"}}
        }

        # 11/29/2017 4pm
        run_time = dateutil.parser.parse("11/29/2017 4:00pm")

        # 5 minute delay
        delay = 300

        # Add job to schedule
        self.schedule.opts.update(job)

        # Postpone the job by 5 minutes
        self.schedule.postpone_job(
            "job1",
            {
                "time": run_time.strftime("%Y-%m-%dT%H:%M:%S"),
                "new_time": (run_time + datetime.timedelta(seconds=delay)).strftime(
                    "%Y-%m-%dT%H:%M:%S"
                ),
            },
        )
        # Run at the original time
        self.schedule.eval(now=run_time)
        ret = self.schedule.job_status("job1")
        self.assertNotIn("_last_run", ret)

        # Run 5 minutes later
        self.schedule.eval(now=run_time + datetime.timedelta(seconds=delay))
        ret = self.schedule.job_status("job1")
        self.assertEqual(ret["_last_run"], run_time + datetime.timedelta(seconds=delay))

        # Run 6 minutes later
        self.schedule.eval(now=run_time + datetime.timedelta(seconds=delay + 1))
        ret = self.schedule.job_status("job1")
        self.assertEqual(ret["_last_run"], run_time + datetime.timedelta(seconds=delay))



# -*- coding: utf-8 -*-
"""
    :codeauthor: Thomas Jackson <jacksontj.89@gmail.com>
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging
import socket
import threading

import salt.config
import salt.exceptions
import salt.ext.tornado.concurrent
import salt.ext.tornado.gen
import salt.ext.tornado.ioloop
import salt.transport.client
import salt.transport.server
import salt.utils.platform
import salt.utils.process
from salt.ext import six
from salt.ext.six.moves import range
from salt.ext.tornado.testing import AsyncTestCase, gen_test
from salt.transport.tcp import (
    SaltMessageClient,
    SaltMessageClientPool,
    TCPPubServerChannel,
)
from saltfactories.utils.ports import get_unused_localhost_port
from tests.support.helpers import flaky, slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf
from tests.unit.transport.mixins import (
    PubChannelMixin,
    ReqChannelMixin,
    run_loop_in_thread,
)

log = logging.getLogger(__name__)


class BaseTCPReqCase(TestCase, AdaptedConfigurationTestCaseMixin):
    """
    Test the req server/client pair
    """

    @classmethod
    def setUpClass(cls):
        if not hasattr(cls, "_handle_payload"):
            return
        ret_port = get_unused_localhost_port()
        publish_port = get_unused_localhost_port()
        tcp_master_pub_port = get_unused_localhost_port()
        tcp_master_pull_port = get_unused_localhost_port()
        tcp_master_publish_pull = get_unused_localhost_port()
        tcp_master_workers = get_unused_localhost_port()
        cls.master_config = cls.get_temp_config(
            "master",
            **{
                "transport": "tcp",
                "auto_accept": True,
                "ret_port": ret_port,
                "publish_port": publish_port,
                "tcp_master_pub_port": tcp_master_pub_port,
                "tcp_master_pull_port": tcp_master_pull_port,
                "tcp_master_publish_pull": tcp_master_publish_pull,
                "tcp_master_workers": tcp_master_workers,
            }
        )

        cls.minion_config = cls.get_temp_config(
            "minion",
            **{
                "transport": "tcp",
                "master_ip": "127.0.0.1",
                "master_port": ret_port,
                "master_uri": "tcp://127.0.0.1:{0}".format(ret_port),
            }
        )

        cls.process_manager = salt.utils.process.ProcessManager(
            name="ReqServer_ProcessManager"
        )

        cls.server_channel = salt.transport.server.ReqServerChannel.factory(
            cls.master_config
        )
        cls.server_channel.pre_fork(cls.process_manager)
        cls.io_loop = salt.ext.tornado.ioloop.IOLoop()
        cls.stop = threading.Event()
        cls.server_channel.post_fork(cls._handle_payload, io_loop=cls.io_loop)
        cls.server_thread = threading.Thread(
            target=run_loop_in_thread, args=(cls.io_loop, cls.stop,),
        )
        cls.server_thread.start()

    @classmethod
    def tearDownClass(cls):
        cls.server_channel.close()
        cls.stop.set()
        cls.server_thread.join()
        cls.process_manager.kill_children()
        del cls.server_channel

    @classmethod
    @salt.ext.tornado.gen.coroutine
    def _handle_payload(cls, payload):
        """
        TODO: something besides echo
        """
        raise salt.ext.tornado.gen.Return((payload, {"fun": "send_clear"}))


@skipIf(salt.utils.platform.is_darwin(), "hanging test suite on MacOS")
class ClearReqTestCases(BaseTCPReqCase, ReqChannelMixin):
    """
    Test all of the clear msg stuff
    """

    def setUp(self):
        self.channel = salt.transport.client.ReqChannel.factory(
            self.minion_config, crypt="clear"
        )

    def tearDown(self):
        self.channel.close()
        del self.channel

    @classmethod
    @salt.ext.tornado.gen.coroutine
    def _handle_payload(cls, payload):
        """
        TODO: something besides echo
        """
        raise salt.ext.tornado.gen.Return((payload, {"fun": "send_clear"}))


@skipIf(salt.utils.platform.is_darwin(), "hanging test suite on MacOS")
class AESReqTestCases(BaseTCPReqCase, ReqChannelMixin):
    def setUp(self):
        self.channel = salt.transport.client.ReqChannel.factory(self.minion_config)

    def tearDown(self):
        self.channel.close()
        del self.channel

    @classmethod
    @salt.ext.tornado.gen.coroutine
    def _handle_payload(cls, payload):
        """
        TODO: something besides echo
        """
        raise salt.ext.tornado.gen.Return((payload, {"fun": "send"}))

    # TODO: make failed returns have a specific framing so we can raise the same exception
    # on encrypted channels
    @flaky
    @slowTest
    def test_badload(self):
        """
        Test a variety of bad requests, make sure that we get some sort of error
        """
        msgs = ["", [], tuple()]
        for msg in msgs:
            with self.assertRaises(salt.exceptions.AuthenticationError):
                ret = self.channel.send(msg)


class BaseTCPPubCase(AsyncTestCase, AdaptedConfigurationTestCaseMixin):
    """
    Test the req server/client pair
    """

    @classmethod
    def setUpClass(cls):
        ret_port = get_unused_localhost_port()
        publish_port = get_unused_localhost_port()
        tcp_master_pub_port = get_unused_localhost_port()
        tcp_master_pull_port = get_unused_localhost_port()
        tcp_master_publish_pull = get_unused_localhost_port()
        tcp_master_workers = get_unused_localhost_port()
        cls.master_config = cls.get_temp_config(
            "master",
            **{
                "transport": "tcp",
                "auto_accept": True,
                "ret_port": ret_port,
                "publish_port": publish_port,
                "tcp_master_pub_port": tcp_master_pub_port,
                "tcp_master_pull_port": tcp_master_pull_port,
                "tcp_master_publish_pull": tcp_master_publish_pull,
                "tcp_master_workers": tcp_master_workers,
            }
        )

        cls.minion_config = cls.get_temp_config(
            "minion",
            **{
                "transport": "tcp",
                "master_ip": "127.0.0.1",
                "auth_timeout": 1,
                "master_port": ret_port,
                "master_uri": "tcp://127.0.0.1:{0}".format(ret_port),
            }
        )

        cls.process_manager = salt.utils.process.ProcessManager(
            name="ReqServer_ProcessManager"
        )

        cls.server_channel = salt.transport.server.PubServerChannel.factory(
            cls.master_config
        )
        cls.server_channel.pre_fork(cls.process_manager)

        # we also require req server for auth
        cls.req_server_channel = salt.transport.server.ReqServerChannel.factory(
            cls.master_config
        )
        cls.req_server_channel.pre_fork(cls.process_manager)
        cls.io_loop = salt.ext.tornado.ioloop.IOLoop()
        cls.stop = threading.Event()
        cls.req_server_channel.post_fork(cls._handle_payload, io_loop=cls.io_loop)
        cls.server_thread = threading.Thread(
            target=run_loop_in_thread, args=(cls.io_loop, cls.stop,),
        )
        cls.server_thread.start()

    @classmethod
    def _handle_payload(cls, payload):
        """
        TODO: something besides echo
        """
        return payload, {"fun": "send_clear"}

    @classmethod
    def tearDownClass(cls):
        cls.req_server_channel.close()
        cls.server_channel.close()
        cls.stop.set()
        cls.server_thread.join()
        cls.process_manager.kill_children()
        del cls.req_server_channel

    def setUp(self):
        super(BaseTCPPubCase, self).setUp()
        self._start_handlers = dict(self.io_loop._handlers)

    def tearDown(self):
        super(BaseTCPPubCase, self).tearDown()
        failures = []
        for k, v in six.iteritems(self.io_loop._handlers):
            if self._start_handlers.get(k) != v:
                failures.append((k, v))
        if failures:
            raise Exception("FDs still attached to the IOLoop: {0}".format(failures))
        del self.channel
        del self._start_handlers


class AsyncTCPPubChannelTest(AsyncTestCase, AdaptedConfigurationTestCaseMixin):
    @slowTest
    def test_connect_publish_port(self):
        """
        test when publish_port is not 4506
        """
        opts = self.get_temp_config("master")
        opts["master_uri"] = ""
        opts["master_ip"] = "127.0.0.1"
        opts["publish_port"] = 1234
        channel = salt.transport.tcp.AsyncTCPPubChannel(opts)
        patch_auth = MagicMock(return_value=True)
        patch_client = MagicMock(spec=SaltMessageClientPool)
        with patch("salt.crypt.AsyncAuth.gen_token", patch_auth), patch(
            "salt.crypt.AsyncAuth.authenticated", patch_auth
        ), patch("salt.transport.tcp.SaltMessageClientPool", patch_client):
            channel.connect()
        assert patch_client.call_args[0][0]["publish_port"] == opts["publish_port"]


@skipIf(True, "Skip until we can devote time to fix this test")
class AsyncPubChannelTest(BaseTCPPubCase, PubChannelMixin):
    """
    Tests around the publish system
    """


class SaltMessageClientPoolTest(AsyncTestCase):
    def setUp(self):
        super(SaltMessageClientPoolTest, self).setUp()
        sock_pool_size = 5
        with patch(
            "salt.transport.tcp.SaltMessageClient.__init__",
            MagicMock(return_value=None),
        ):
            self.message_client_pool = SaltMessageClientPool(
                {"sock_pool_size": sock_pool_size}, args=({}, "", 0)
            )
        self.original_message_clients = self.message_client_pool.message_clients
        self.message_client_pool.message_clients = [
            MagicMock() for _ in range(sock_pool_size)
        ]

    def tearDown(self):
        with patch(
            "salt.transport.tcp.SaltMessageClient.close", MagicMock(return_value=None)
        ):
            del self.original_message_clients
        super(SaltMessageClientPoolTest, self).tearDown()

    def test_send(self):
        for message_client_mock in self.message_client_pool.message_clients:
            message_client_mock.send_queue = [0, 0, 0]
            message_client_mock.send.return_value = []
        self.assertEqual([], self.message_client_pool.send())
        self.message_client_pool.message_clients[2].send_queue = [0]
        self.message_client_pool.message_clients[2].send.return_value = [1]
        self.assertEqual([1], self.message_client_pool.send())

    def test_write_to_stream(self):
        for message_client_mock in self.message_client_pool.message_clients:
            message_client_mock.send_queue = [0, 0, 0]
            message_client_mock._stream.write.return_value = []
        self.assertEqual([], self.message_client_pool.write_to_stream(""))
        self.message_client_pool.message_clients[2].send_queue = [0]
        self.message_client_pool.message_clients[2]._stream.write.return_value = [1]
        self.assertEqual([1], self.message_client_pool.write_to_stream(""))

    def test_close(self):
        self.message_client_pool.close()
        self.assertEqual([], self.message_client_pool.message_clients)

    def test_on_recv(self):
        for message_client_mock in self.message_client_pool.message_clients:
            message_client_mock.on_recv.return_value = None
        self.message_client_pool.on_recv()
        for message_client_mock in self.message_client_pool.message_clients:
            self.assertTrue(message_client_mock.on_recv.called)

    def test_connect_all(self):
        @gen_test
        def test_connect(self):
            yield self.message_client_pool.connect()

        for message_client_mock in self.message_client_pool.message_clients:
            future = salt.ext.tornado.concurrent.Future()
            future.set_result("foo")
            message_client_mock.connect.return_value = future

        self.assertIsNone(test_connect(self))

    def test_connect_partial(self):
        @gen_test(timeout=0.1)
        def test_connect(self):
            yield self.message_client_pool.connect()

        for idx, message_client_mock in enumerate(
            self.message_client_pool.message_clients
        ):
            future = salt.ext.tornado.concurrent.Future()
            if idx % 2 == 0:
                future.set_result("foo")
            message_client_mock.connect.return_value = future

        with self.assertRaises(salt.ext.tornado.ioloop.TimeoutError):
            test_connect(self)


class SaltMessageClientCleanupTest(TestCase, AdaptedConfigurationTestCaseMixin):
    def setUp(self):
        self.listen_on = "127.0.0.1"
        self.port = get_unused_localhost_port()
        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.sock.bind((self.listen_on, self.port))
        self.sock.listen(1)

    def tearDown(self):
        self.sock.close()
        del self.sock

    def test_message_client(self):
        """
        test message client cleanup on close
        """
        orig_loop = salt.ext.tornado.ioloop.IOLoop()
        orig_loop.make_current()
        opts = self.get_temp_config("master")
        client = SaltMessageClient(opts, self.listen_on, self.port)

        # Mock the io_loop's stop method so we know when it has been called.
        orig_loop.real_stop = orig_loop.stop
        orig_loop.stop_called = False

        def stop(*args, **kwargs):
            orig_loop.stop_called = True
            orig_loop.real_stop()

        orig_loop.stop = stop
        try:
            assert client.io_loop == orig_loop
            client.io_loop.run_sync(client.connect)

            # Ensure we are testing the _read_until_future and io_loop teardown
            assert client._stream is not None
            assert client._read_until_future is not None
            assert orig_loop.stop_called is True

            # The run_sync call will set stop_called, reset it
            orig_loop.stop_called = False
            client.close()

            # Stop should be called again, client's io_loop should be None
            assert orig_loop.stop_called is True
            assert client.io_loop is None
        finally:
            orig_loop.stop = orig_loop.real_stop
            del orig_loop.real_stop
            del orig_loop.stop_called


class TCPPubServerChannelTest(TestCase, AdaptedConfigurationTestCaseMixin):
    @patch("salt.master.SMaster.secrets")
    @patch("salt.crypt.Crypticle")
    @patch("salt.utils.asynchronous.SyncWrapper")
    def test_publish_filtering(self, sync_wrapper, crypticle, secrets):
        opts = self.get_temp_config("master")
        opts["sign_pub_messages"] = False
        channel = TCPPubServerChannel(opts)

        wrap = MagicMock()
        crypt = MagicMock()
        crypt.dumps.return_value = {"test": "value"}

        secrets.return_value = {"aes": {"secret": None}}
        crypticle.return_value = crypt
        sync_wrapper.return_value = wrap

        # try simple publish with glob tgt_type
        channel.publish({"test": "value", "tgt_type": "glob", "tgt": "*"})
        payload = wrap.send.call_args[0][0]

        # verify we send it without any specific topic
        assert "topic_lst" not in payload

        # try simple publish with list tgt_type
        channel.publish({"test": "value", "tgt_type": "list", "tgt": ["minion01"]})
        payload = wrap.send.call_args[0][0]

        # verify we send it with correct topic
        assert "topic_lst" in payload
        self.assertEqual(payload["topic_lst"], ["minion01"])

        # try with syndic settings
        opts["order_masters"] = True
        channel.publish({"test": "value", "tgt_type": "list", "tgt": ["minion01"]})
        payload = wrap.send.call_args[0][0]

        # verify we send it without topic for syndics
        assert "topic_lst" not in payload

    @patch("salt.utils.minions.CkMinions.check_minions")
    @patch("salt.master.SMaster.secrets")
    @patch("salt.crypt.Crypticle")
    @patch("salt.utils.asynchronous.SyncWrapper")
    def test_publish_filtering_str_list(
        self, sync_wrapper, crypticle, secrets, check_minions
    ):
        opts = self.get_temp_config("master")
        opts["sign_pub_messages"] = False
        channel = TCPPubServerChannel(opts)

        wrap = MagicMock()
        crypt = MagicMock()
        crypt.dumps.return_value = {"test": "value"}

        secrets.return_value = {"aes": {"secret": None}}
        crypticle.return_value = crypt
        sync_wrapper.return_value = wrap
        check_minions.return_value = {"minions": ["minion02"]}

        # try simple publish with list tgt_type
        channel.publish({"test": "value", "tgt_type": "list", "tgt": "minion02"})
        payload = wrap.send.call_args[0][0]

        # verify we send it with correct topic
        assert "topic_lst" in payload
        self.assertEqual(payload["topic_lst"], ["minion02"])

        # verify it was correctly calling check_minions
        check_minions.assert_called_with("minion02", tgt_type="list")



# -*- coding: utf-8 -*-
"""
Return salt data via email

The following fields can be set in the minion conf file. Fields are optional
unless noted otherwise.

* ``from`` (required) The name/address of the email sender.
* ``to`` (required) The names/addresses of the email recipients;
    comma-delimited. For example: ``you@example.com,someoneelse@example.com``.
* ``host`` (required) The SMTP server hostname or address.
* ``port`` The SMTP server port; defaults to ``25``.
* ``username`` The username used to authenticate to the server. If specified a
    password is also required. It is recommended but not required to also use
    TLS with this option.
* ``password`` The password used to authenticate to the server.
* ``tls`` Whether to secure the connection using TLS; defaults to ``False``
* ``subject`` The email subject line.
* ``fields`` Which fields from the returned data to include in the subject line
    of the email; comma-delimited. For example: ``id,fun``. Please note, *the
    subject line is not encrypted*.
* ``gpgowner`` A user's :file:`~/.gpg` directory. This must contain a gpg
    public key matching the address the mail is sent to. If left unset, no
    encryption will be used. Requires :program:`python-gnupg` to be installed.
* ``template`` The path to a file to be used as a template for the email body.
* ``renderer`` A Salt renderer, or render-pipe, to use to render the email
    template. Default ``jinja``.

Below is an example of the above settings in a Salt Minion configuration file:

.. code-block:: yaml

    smtp.from: me@example.net
    smtp.to: you@example.com
    smtp.host: localhost
    smtp.port: 1025

Alternative configuration values can be used by prefacing the configuration.
Any values not found in the alternative configuration will be pulled from
the default location. For example:

.. code-block:: yaml

    alternative.smtp.username: saltdev
    alternative.smtp.password: saltdev
    alternative.smtp.tls: True

To use the SMTP returner, append '--return smtp' to the ``salt`` command.

.. code-block:: bash

    salt '*' test.ping --return smtp

To use the alternative configuration, append '--return_config alternative' to the ``salt`` command.

.. versionadded:: 2015.5.0

.. code-block:: bash

    salt '*' test.ping --return smtp --return_config alternative

To override individual configuration items, append --return_kwargs '{"key:": "value"}' to the
``salt`` command.

.. versionadded:: 2016.3.0

.. code-block:: bash

    salt '*' test.ping --return smtp --return_kwargs '{"to": "user@domain.com"}'

An easy way to test the SMTP returner is to use the development SMTP server
built into Python. The command below will start a single-threaded SMTP server
that prints any email it receives to the console.

.. code-block:: python

    python -m smtpd -n -c DebuggingServer localhost:1025

.. versionadded:: 2016.11.0

It is possible to send emails with selected Salt events by configuring ``event_return`` option
for Salt Master. For example:

.. code-block:: yaml

    event_return: smtp

    event_return_whitelist:
      - salt/key

    smtp.from: me@example.net
    smtp.to: you@example.com
    smtp.host: localhost
    smtp.subject: 'Salt Master {{act}}ed key from Minion ID: {{id}}'
    smtp.template: /srv/salt/templates/email.j2

Also you need to create additional file ``/srv/salt/templates/email.j2`` with email body template:

.. code-block:: yaml

    act: {{act}}
    id: {{id}}
    result: {{result}}

This configuration enables Salt Master to send an email when accepting or rejecting minions keys.
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import smtplib
from email.utils import formatdate

import salt.loader
import salt.returners
import salt.utils.jid

# Import Salt libs
from salt.ext import six
from salt.template import compile_template

try:
    import gnupg

    HAS_GNUPG = True
except ImportError:
    HAS_GNUPG = False


log = logging.getLogger(__name__)

__virtualname__ = "smtp"


def __virtual__():
    return __virtualname__


def _get_options(ret=None):
    """
    Get the SMTP options from salt.
    """
    attrs = {
        "from": "from",
        "to": "to",
        "host": "host",
        "port": "port",
        "username": "username",
        "password": "password",
        "subject": "subject",
        "gpgowner": "gpgowner",
        "fields": "fields",
        "tls": "tls",
        "renderer": "renderer",
        "template": "template",
    }

    _options = salt.returners.get_returner_options(
        __virtualname__, ret, attrs, __salt__=__salt__, __opts__=__opts__
    )
    return _options


def returner(ret):
    """
    Send an email with the data
    """

    _options = _get_options(ret)
    from_addr = _options.get("from")
    to_addrs = _options.get("to").split(",")
    host = _options.get("host")
    port = _options.get("port")
    user = _options.get("username")
    passwd = _options.get("password")
    subject = _options.get("subject") or "Email from Salt"
    gpgowner = _options.get("gpgowner")
    fields = _options.get("fields").split(",") if "fields" in _options else []
    smtp_tls = _options.get("tls")

    renderer = _options.get("renderer") or "jinja"
    rend = salt.loader.render(__opts__, {})
    blacklist = __opts__.get("renderer_blacklist")
    whitelist = __opts__.get("renderer_whitelist")

    if not port:
        port = 25
    log.debug("SMTP port has been set to %s", port)

    for field in fields:
        if field in ret:
            subject += " {0}".format(ret[field])
    subject = compile_template(
        ":string:", rend, renderer, blacklist, whitelist, input_data=subject, **ret
    )
    if isinstance(subject, six.moves.StringIO):
        subject = subject.read()
    log.debug("smtp_return: Subject is '%s'", subject)

    template = _options.get("template")
    if template:
        content = compile_template(
            template, rend, renderer, blacklist, whitelist, **ret
        )
    else:
        template = (
            "id: {{id}}\r\n"
            "function: {{fun}}\r\n"
            "function args: {{fun_args}}\r\n"
            "jid: {{jid}}\r\n"
            "return: {{return}}\r\n"
        )
        content = compile_template(
            ":string:", rend, renderer, blacklist, whitelist, input_data=template, **ret
        )

    if gpgowner:
        if HAS_GNUPG:
            gpg = gnupg.GPG(
                gnupghome=os.path.expanduser("~{0}/.gnupg".format(gpgowner)),
                options=["--trust-model always"],
            )
            encrypted_data = gpg.encrypt(content, to_addrs)
            if encrypted_data.ok:
                log.debug("smtp_return: Encryption successful")
                content = six.text_type(encrypted_data)
            else:
                log.error(
                    "smtp_return: Encryption failed, only an error message will be sent"
                )
                content = "Encryption failed, the return data was not sent.\r\n\r\n{0}\r\n{1}".format(
                    encrypted_data.status, encrypted_data.stderr
                )
        else:
            log.error(
                "gnupg python module is required in order to user gpgowner in smtp returner ; ignoring gpgowner configuration for now"
            )
    if isinstance(content, six.moves.StringIO):
        content = content.read()

    message = (
        "From: {0}\r\n" "To: {1}\r\n" "Date: {2}\r\n" "Subject: {3}\r\n" "\r\n" "{4}"
    ).format(
        from_addr, ", ".join(to_addrs), formatdate(localtime=True), subject, content
    )

    log.debug("smtp_return: Connecting to the server...")
    server = smtplib.SMTP(host, int(port))
    if smtp_tls is True:
        server.starttls()
        log.debug("smtp_return: TLS enabled")
    if user and passwd:
        server.login(user, passwd)
        log.debug("smtp_return: Authenticated")
    # enable logging SMTP session after the login credentials were passed
    server.set_debuglevel(1)
    server.sendmail(from_addr, to_addrs, message)
    log.debug("smtp_return: Message sent.")
    server.quit()


def prep_jid(nocache=False, passed_jid=None):  # pylint: disable=unused-argument
    """
    Do any work necessary to prepare a JID, including sending a custom id
    """
    return passed_jid if passed_jid is not None else salt.utils.jid.gen_jid(__opts__)


def event_return(events):
    """
    Return event data via SMTP
    """

    for event in events:
        ret = event.get("data", False)

        if ret:
            returner(ret)



# -*- coding: utf-8 -*-
"""
Runner for SmartOS minions control vmadm
"""
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import salt libs
import salt.client
from salt.exceptions import SaltClientError

# Import 3rd party libs
from salt.ext import six
from salt.utils.odict import OrderedDict

# Function aliases
__func_alias__ = {"list_vms": "list"}

# Define the module's virtual name
__virtualname__ = "vmadm"


def __virtual__():
    """
    Provides vmadm runner
    """
    # NOTE: always load vmadm runner
    #       we could check using test.ping + a grain
    #       match, but doing this on master startup is
    #       not acceptable
    return __virtualname__


def _action(action="get", search=None, one=True, force=False):
    """
    Multi action helper for start, stop, get, ...
    """
    vms = {}
    matched_vms = []
    client = salt.client.get_local_client(__opts__["conf_file"])

    ## lookup vms
    try:
        vmadm_args = {}
        vmadm_args["order"] = "uuid,alias,hostname,state"
        if "=" in search:
            vmadm_args["search"] = search
        for cn in client.cmd_iter(
            "G@virtual:physical and G@os:smartos",
            "vmadm.list",
            kwarg=vmadm_args,
            tgt_type="compound",
        ):
            if not cn:
                continue
            node = next(six.iterkeys(cn))
            if (
                not isinstance(cn[node], dict)
                or "ret" not in cn[node]
                or not isinstance(cn[node]["ret"], dict)
            ):
                continue
            for vm in cn[node]["ret"]:
                vmcfg = cn[node]["ret"][vm]
                vmcfg["node"] = node
                vms[vm] = vmcfg
    except SaltClientError as client_error:
        pass

    ## check if we have vms
    if len(vms) == 0:
        return {"Error": "No vms found."}

    ## simple search
    if "=" not in search:
        loop_pass = 0
        while loop_pass < 3:
            ## each pass will try a different field
            if loop_pass == 0:
                field = "uuid"
            elif loop_pass == 1:
                field = "hostname"
            else:
                field = "alias"

            ## loop vms and try to match
            for vm in vms:
                if field == "uuid" and vm == search:
                    matched_vms.append(vm)
                    break  # exit for on uuid match (max = 1)
                elif field in vms[vm] and vms[vm][field] == search:
                    matched_vms.append(vm)

            ## exit on match(es) or try again
            if len(matched_vms) > 0:
                break
            else:
                loop_pass += 1
    else:
        for vm in vms:
            matched_vms.append(vm)

    ## check if we have vms
    if len(matched_vms) == 0:
        return {"Error": "No vms matched."}

    ## multiple allowed?
    if one and len(matched_vms) > 1:
        return {
            "Error": "Matched {0} vms, only one allowed!".format(len(matched_vms)),
            "Matches": matched_vms,
        }

    ## perform action
    ret = {}
    if action in ["start", "stop", "reboot", "get"]:
        for vm in matched_vms:
            vmadm_args = {"key": "uuid", "vm": vm}
            try:
                for vmadm_res in client.cmd_iter(
                    vms[vm]["node"], "vmadm.{0}".format(action), kwarg=vmadm_args
                ):
                    if not vmadm_res:
                        continue
                    if vms[vm]["node"] in vmadm_res:
                        ret[vm] = vmadm_res[vms[vm]["node"]]["ret"]
            except SaltClientError as client_error:
                ret[vm] = False
    elif action in ["is_running"]:
        ret = True
        for vm in matched_vms:
            if vms[vm]["state"] != "running":
                ret = False
                break
    return ret


def nodes(verbose=False):
    """
    List all compute nodes

    verbose : boolean
        print additional information about the node
        e.g. platform version, hvm capable, ...

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.nodes
        salt-run vmadm.nodes verbose=True
    """
    ret = {} if verbose else []
    client = salt.client.get_local_client(__opts__["conf_file"])

    ## get list of nodes
    try:
        for cn in client.cmd_iter(
            "G@virtual:physical and G@os:smartos", "grains.items", tgt_type="compound"
        ):
            if not cn:
                continue
            node = next(six.iterkeys(cn))
            if (
                not isinstance(cn[node], dict)
                or "ret" not in cn[node]
                or not isinstance(cn[node]["ret"], dict)
            ):
                continue
            if verbose:
                ret[node] = {}
                ret[node]["version"] = {}
                ret[node]["version"]["platform"] = cn[node]["ret"]["osrelease"]
                if "computenode_sdc_version" in cn[node]["ret"]:
                    ret[node]["version"]["sdc"] = cn[node]["ret"][
                        "computenode_sdc_version"
                    ]
                ret[node]["vms"] = {}
                if (
                    "computenode_vm_capable" in cn[node]["ret"]
                    and cn[node]["ret"]["computenode_vm_capable"]
                    and "computenode_vm_hw_virt" in cn[node]["ret"]
                ):
                    ret[node]["vms"]["hw_cap"] = cn[node]["ret"][
                        "computenode_vm_hw_virt"
                    ]
                else:
                    ret[node]["vms"]["hw_cap"] = False
                if "computenode_vms_running" in cn[node]["ret"]:
                    ret[node]["vms"]["running"] = cn[node]["ret"][
                        "computenode_vms_running"
                    ]
            else:
                ret.append(node)
    except SaltClientError as client_error:
        return "{0}".format(client_error)

    if not verbose:
        ret.sort()
    return ret


def list_vms(search=None, verbose=False):
    """
    List all vms

    search : string
        filter vms, see the execution module
    verbose : boolean
        print additional information about the vm

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.list
        salt-run vmadm.list search='type=KVM'
        salt-run vmadm.list verbose=True
    """
    ret = OrderedDict() if verbose else []
    client = salt.client.get_local_client(__opts__["conf_file"])
    try:
        vmadm_args = {}
        vmadm_args["order"] = "uuid,alias,hostname,state,type,cpu_cap,vcpus,ram"
        if search:
            vmadm_args["search"] = search
        for cn in client.cmd_iter(
            "G@virtual:physical and G@os:smartos",
            "vmadm.list",
            kwarg=vmadm_args,
            tgt_type="compound",
        ):
            if not cn:
                continue
            node = next(six.iterkeys(cn))
            if (
                not isinstance(cn[node], dict)
                or "ret" not in cn[node]
                or not isinstance(cn[node]["ret"], dict)
            ):
                continue
            for vm in cn[node]["ret"]:
                vmcfg = cn[node]["ret"][vm]
                if verbose:
                    ret[vm] = OrderedDict()
                    ret[vm]["hostname"] = vmcfg["hostname"]
                    ret[vm]["alias"] = vmcfg["alias"]
                    ret[vm]["computenode"] = node
                    ret[vm]["state"] = vmcfg["state"]
                    ret[vm]["resources"] = OrderedDict()
                    ret[vm]["resources"]["memory"] = vmcfg["ram"]
                    if vmcfg["type"] == "KVM":
                        ret[vm]["resources"]["cpu"] = "{0:.2f}".format(
                            int(vmcfg["vcpus"])
                        )
                    else:
                        if vmcfg["cpu_cap"] != "":
                            ret[vm]["resources"]["cpu"] = "{0:.2f}".format(
                                int(vmcfg["cpu_cap"]) / 100
                            )
                else:
                    ret.append(vm)
    except SaltClientError as client_error:
        return "{0}".format(client_error)

    if not verbose:
        ret = sorted(ret)

    return ret


def start(search, one=True):
    """
    Start one or more vms

    search : string
        filter vms, see the execution module.
    one : boolean
        start only one vm

    .. note::
        If the search parameter does not contain an equal (=) symbol it will be
        assumed it will be tried as uuid, hostname, and alias.

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.start 91244bba-1146-e4ec-c07e-e825e0223aa9
        salt-run vmadm.start search='alias=jiska'
        salt-run vmadm.start search='type=KVM' one=False
    """
    return _action("start", search, one)


def stop(search, one=True):
    """
    Stop one or more vms

    search : string
        filter vms, see the execution module.
    one : boolean
        stop only one vm

    .. note::
        If the search parameter does not contain an equal (=) symbol it will be
        assumed it will be tried as uuid, hostname, and alias.

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.stop 91244bba-1146-e4ec-c07e-e825e0223aa9
        salt-run vmadm.stop search='alias=jody'
        salt-run vmadm.stop search='type=KVM' one=False
    """
    return _action("stop", search, one)


def reboot(search, one=True, force=False):
    """
    Reboot one or more vms

    search : string
        filter vms, see the execution module.
    one : boolean
        reboot only one vm
    force : boolean
        force reboot, faster but no graceful shutdown

    .. note::
        If the search parameter does not contain an equal (=) symbol it will be
        assumed it will be tried as uuid, hostname, and alias.

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.reboot 91244bba-1146-e4ec-c07e-e825e0223aa9
        salt-run vmadm.reboot search='alias=marije'
        salt-run vmadm.reboot search='type=KVM' one=False
    """
    return _action("reboot", search, one, force)


def get(search, one=True):
    """
    Return information for vms

    search : string
        filter vms, see the execution module.
    one : boolean
        return only one vm

    .. note::
        If the search parameter does not contain an equal (=) symbol it will be
        assumed it will be tried as uuid, hostname, and alias.

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.get 91244bba-1146-e4ec-c07e-e825e0223aa9
        salt-run vmadm.get search='alias=saskia'
    """
    return _action("get", search, one)


def is_running(search):
    """
    Return true if vm is running

    search : string
        filter vms, see the execution module.

    .. note::
        If the search parameter does not contain an equal (=) symbol it will be
        assumed it will be tried as uuid, hostname, and alias.

    .. note::
        If multiple vms are matched, the result will be true of ALL vms are running

    CLI Example:

    .. code-block:: bash

        salt-run vmadm.is_running 91244bba-1146-e4ec-c07e-e825e0223aa9
        salt-run vmadm.is_running search='alias=julia'
    """
    return _action("is_running", search, False)


# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4



# -*- coding: utf-8 -*-
"""
Unit tests for salt.utils.templates.py
"""

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import sys

import salt.utils.files

# Import Salt libs
import salt.utils.templates

# Import Salt Testing Libs
from tests.support.helpers import with_tempdir
from tests.support.unit import TestCase, skipIf

try:
    import Cheetah as _

    HAS_CHEETAH = True
except ImportError:
    HAS_CHEETAH = False

log = logging.getLogger(__name__)


### Here we go!
class RenderTestCase(TestCase):
    def setUp(self):
        # Default context for salt.utils.templates.render_*_tmpl to work
        self.context = {
            "opts": {"cachedir": "/D", "__cli": "salt"},
            "saltenv": None,
        }

    ### Tests for Jinja (whitespace-friendly)
    def test_render_jinja_sanity(self):
        tmpl = """OK"""
        res = salt.utils.templates.render_jinja_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    def test_render_jinja_evaluate(self):
        tmpl = """{{ "OK" }}"""
        res = salt.utils.templates.render_jinja_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    def test_render_jinja_evaluate_multi(self):
        tmpl = """{% if 1 -%}OK{%- endif %}"""
        res = salt.utils.templates.render_jinja_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    def test_render_jinja_variable(self):
        tmpl = """{{ var }}"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_jinja_tmpl(tmpl, ctx)
        self.assertEqual(res, "OK")

    ### Tests for mako template
    def test_render_mako_sanity(self):
        tmpl = """OK"""
        res = salt.utils.templates.render_mako_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    def test_render_mako_evaluate(self):
        tmpl = """${ "OK" }"""
        res = salt.utils.templates.render_mako_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    def test_render_mako_evaluate_multi(self):
        tmpl = """
        % if 1:
        OK
        % endif
        """
        res = salt.utils.templates.render_mako_tmpl(tmpl, dict(self.context))
        stripped = res.strip()
        self.assertEqual(stripped, "OK")

    def test_render_mako_variable(self):
        tmpl = """${ var }"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_mako_tmpl(tmpl, ctx)
        self.assertEqual(res, "OK")

    ### Tests for wempy template
    @skipIf(
        sys.version_info > (3,),
        "The wempy module is currently unsupported under Python3",
    )
    def test_render_wempy_sanity(self):
        tmpl = """OK"""
        res = salt.utils.templates.render_wempy_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    @skipIf(
        sys.version_info > (3,),
        "The wempy module is currently unsupported under Python3",
    )
    def test_render_wempy_evaluate(self):
        tmpl = """{{="OK"}}"""
        res = salt.utils.templates.render_wempy_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    @skipIf(
        sys.version_info > (3,),
        "The wempy module is currently unsupported under Python3",
    )
    def test_render_wempy_evaluate_multi(self):
        tmpl = """{{if 1:}}OK{{pass}}"""
        res = salt.utils.templates.render_wempy_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    @skipIf(
        sys.version_info > (3,),
        "The wempy module is currently unsupported under Python3",
    )
    def test_render_wempy_variable(self):
        tmpl = """{{=var}}"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_wempy_tmpl(tmpl, ctx)
        self.assertEqual(res, "OK")

    ### Tests for genshi template (xml-based)
    def test_render_genshi_sanity(self):
        tmpl = """<RU>OK</RU>"""
        res = salt.utils.templates.render_genshi_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "<RU>OK</RU>")

    def test_render_genshi_evaluate(self):
        tmpl = """<RU>${ "OK" }</RU>"""
        res = salt.utils.templates.render_genshi_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "<RU>OK</RU>")

    def test_render_genshi_evaluate_condition(self):
        tmpl = """<RU xmlns:py="http://genshi.edgewall.org/" py:if="1">OK</RU>"""
        res = salt.utils.templates.render_genshi_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "<RU>OK</RU>")

    def test_render_genshi_variable(self):
        tmpl = """<RU>$var</RU>"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_genshi_tmpl(tmpl, ctx)
        self.assertEqual(res, "<RU>OK</RU>")

    def test_render_genshi_variable_replace(self):
        tmpl = """<RU xmlns:py="http://genshi.edgewall.org/" py:content="var">not ok</RU>"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_genshi_tmpl(tmpl, ctx)
        self.assertEqual(res, "<RU>OK</RU>")

    ### Tests for cheetah template (line-oriented and xml-friendly)
    @skipIf(not HAS_CHEETAH, "The Cheetah Python module is missing.")
    def test_render_cheetah_sanity(self):
        tmpl = """OK"""
        res = salt.utils.templates.render_cheetah_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    @skipIf(not HAS_CHEETAH, "The Cheetah Python module is missing.")
    def test_render_cheetah_evaluate(self):
        tmpl = """<%="OK"%>"""
        res = salt.utils.templates.render_cheetah_tmpl(tmpl, dict(self.context))
        self.assertEqual(res, "OK")

    @skipIf(not HAS_CHEETAH, "The Cheetah Python module is missing.")
    def test_render_cheetah_evaluate_xml(self):
        tmpl = """
        <% if 1: %>
        OK
        <% pass %>
        """
        res = salt.utils.templates.render_cheetah_tmpl(tmpl, dict(self.context))
        stripped = res.strip()
        self.assertEqual(stripped, "OK")

    @skipIf(not HAS_CHEETAH, "The Cheetah Python module is missing.")
    def test_render_cheetah_evaluate_text(self):
        tmpl = """
        #if 1
        OK
        #end if
        """

        res = salt.utils.templates.render_cheetah_tmpl(tmpl, dict(self.context))
        stripped = res.strip()
        self.assertEqual(stripped, "OK")

    @skipIf(not HAS_CHEETAH, "The Cheetah Python module is missing.")
    def test_render_cheetah_variable(self):
        tmpl = """$var"""

        ctx = dict(self.context)
        ctx["var"] = "OK"
        res = salt.utils.templates.render_cheetah_tmpl(tmpl, ctx)
        self.assertEqual(res.strip(), "OK")


class MockRender(object):
    def __call__(self, tplstr, context, tmplpath=None):
        self.tplstr = tplstr
        self.context = context
        self.tmplpath = tmplpath
        return tplstr


class WrapRenderTestCase(TestCase):
    @with_tempdir()
    def test_wrap_issue_56119_a(self, tempdir):
        slsfile = os.path.join(tempdir, "foo")
        with salt.utils.files.fopen(slsfile, "w") as fp:
            fp.write("{{ slspath }}")
        context = {"opts": {}, "saltenv": "base", "sls": "foo.bar"}
        render = MockRender()
        wrapped = salt.utils.templates.wrap_tmpl_func(render)
        res = wrapped(slsfile, context=context, tmplpath="/tmp/foo/bar/init.sls")
        assert render.context["slspath"] == "foo/bar", render.context["slspath"]
        assert render.context["tpldir"] == "foo/bar", render.context["tpldir"]

    @with_tempdir()
    def test_wrap_issue_56119_b(self, tempdir):
        slsfile = os.path.join(tempdir, "foo")
        with salt.utils.files.fopen(slsfile, "w") as fp:
            fp.write("{{ slspath }}")
        context = {"opts": {}, "saltenv": "base", "sls": "foo.bar.bang"}
        render = MockRender()
        wrapped = salt.utils.templates.wrap_tmpl_func(render)
        res = wrapped(slsfile, context=context, tmplpath="/tmp/foo/bar/bang.sls")
        assert render.context["slspath"] == "foo/bar", render.context["slspath"]
        assert render.context["tpldir"] == "foo/bar", render.context["tpldir"]



# -*- coding: utf-8 -*-
"""
Manage transport commands via ssh
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os

# Import python libs
import re
import subprocess
import sys
import time

# Import salt libs
import salt.defaults.exitcodes
import salt.utils.json
import salt.utils.nb_popen
import salt.utils.vt
from salt.ext import six

log = logging.getLogger(__name__)

SSH_PASSWORD_PROMPT_RE = re.compile(r"(?:.*)[Pp]assword(?: for .*)?:", re.M)
KEY_VALID_RE = re.compile(r".*\(yes\/no\).*")
SSH_PRIVATE_KEY_PASSWORD_PROMPT_RE = re.compile(r"Enter passphrase for key", re.M)

# Keep these in sync with ./__init__.py
RSTR = "_edbc7885e4f9aac9b83b35999b68d015148caf467b78fa39c05f669c0ff89878"
RSTR_RE = re.compile(r"(?:^|\r?\n)" + RSTR + r"(?:\r?\n|$)")


class NoPasswdError(Exception):
    pass


class KeyAcceptError(Exception):
    pass


def gen_key(path):
    """
    Generate a key for use with salt-ssh
    """
    cmd = 'ssh-keygen -P "" -f {0} -t rsa -q'.format(path)
    if not os.path.isdir(os.path.dirname(path)):
        os.makedirs(os.path.dirname(path))
    subprocess.call(cmd, shell=True)


def gen_shell(opts, **kwargs):
    """
    Return the correct shell interface for the target system
    """
    if kwargs["winrm"]:
        try:
            import saltwinshell

            shell = saltwinshell.Shell(opts, **kwargs)
        except ImportError:
            log.error("The saltwinshell library is not available")
            sys.exit(salt.defaults.exitcodes.EX_GENERIC)
    else:
        shell = Shell(opts, **kwargs)
    return shell


class Shell(object):
    """
    Create a shell connection object to encapsulate ssh executions
    """

    def __init__(
        self,
        opts,
        host,
        user=None,
        port=None,
        passwd=None,
        priv=None,
        priv_passwd=None,
        timeout=None,
        sudo=False,
        tty=False,
        mods=None,
        identities_only=False,
        sudo_user=None,
        remote_port_forwards=None,
        winrm=False,
        ssh_options=None,
    ):
        self.opts = opts
        # ssh <ipv6>, but scp [<ipv6]:/path
        self.host = host.strip("[]")
        self.user = user
        self.port = port
        self.passwd = six.text_type(passwd) if passwd else passwd
        self.priv = priv
        self.priv_passwd = priv_passwd
        self.timeout = timeout
        self.sudo = sudo
        self.tty = tty
        self.mods = mods
        self.identities_only = identities_only
        self.remote_port_forwards = remote_port_forwards
        self.ssh_options = "" if ssh_options is None else ssh_options

    def get_error(self, errstr):
        """
        Parse out an error and return a targeted error string
        """
        for line in errstr.split("\n"):
            if line.startswith("ssh:"):
                return line
            if line.startswith("Pseudo-terminal"):
                continue
            if "to the list of known hosts." in line:
                continue
            return line
        return errstr

    def _key_opts(self):
        """
        Return options for the ssh command base for Salt to call
        """
        options = [
            "KbdInteractiveAuthentication=no",
        ]
        if self.passwd:
            options.append("PasswordAuthentication=yes")
        else:
            options.append("PasswordAuthentication=no")
        if self.opts.get("_ssh_version", (0,)) > (4, 9):
            options.append("GSSAPIAuthentication=no")
        options.append("ConnectTimeout={0}".format(self.timeout))
        if self.opts.get("ignore_host_keys"):
            options.append("StrictHostKeyChecking=no")
        if self.opts.get("no_host_keys"):
            options.extend(["StrictHostKeyChecking=no", "UserKnownHostsFile=/dev/null"])
        known_hosts = self.opts.get("known_hosts_file")
        if known_hosts and os.path.isfile(known_hosts):
            options.append("UserKnownHostsFile={0}".format(known_hosts))
        if self.port:
            options.append("Port={0}".format(self.port))
        if self.priv and self.priv != "agent-forwarding":
            options.append("IdentityFile={0}".format(self.priv))
        if self.user:
            options.append("User={0}".format(self.user))
        if self.identities_only:
            options.append("IdentitiesOnly=yes")

        ret = []
        for option in options:
            ret.append("-o {0} ".format(option))
        return "".join(ret)

    def _passwd_opts(self):
        """
        Return options to pass to ssh
        """
        # TODO ControlMaster does not work without ControlPath
        # user could take advantage of it if they set ControlPath in their
        # ssh config.  Also, ControlPersist not widely available.
        options = [
            "ControlMaster=auto",
            "StrictHostKeyChecking=no",
        ]
        if self.opts["_ssh_version"] > (4, 9):
            options.append("GSSAPIAuthentication=no")
        options.append("ConnectTimeout={0}".format(self.timeout))
        if self.opts.get("ignore_host_keys"):
            options.append("StrictHostKeyChecking=no")
        if self.opts.get("no_host_keys"):
            options.extend(["StrictHostKeyChecking=no", "UserKnownHostsFile=/dev/null"])

        if self.passwd:
            options.extend(["PasswordAuthentication=yes", "PubkeyAuthentication=yes"])
        else:
            options.extend(
                [
                    "PasswordAuthentication=no",
                    "PubkeyAuthentication=yes",
                    "KbdInteractiveAuthentication=no",
                    "ChallengeResponseAuthentication=no",
                    "BatchMode=yes",
                ]
            )
        if self.port:
            options.append("Port={0}".format(self.port))
        if self.user:
            options.append("User={0}".format(self.user))
        if self.identities_only:
            options.append("IdentitiesOnly=yes")

        ret = []
        for option in options:
            ret.append("-o {0} ".format(option))
        return "".join(ret)

    def _ssh_opts(self):
        return " ".join(["-o {0}".format(opt) for opt in self.ssh_options])

    def _copy_id_str_old(self):
        """
        Return the string to execute ssh-copy-id
        """
        if self.passwd:
            # Using single quotes prevents shell expansion and
            # passwords containing '$'
            return "{0} {1} '{2} -p {3} {4} {5}@{6}'".format(
                "ssh-copy-id",
                "-i {0}.pub".format(self.priv),
                self._passwd_opts(),
                self.port,
                self._ssh_opts(),
                self.user,
                self.host,
            )
        return None

    def _copy_id_str_new(self):
        """
        Since newer ssh-copy-id commands ingest option differently we need to
        have two commands
        """
        if self.passwd:
            # Using single quotes prevents shell expansion and
            # passwords containing '$'
            return "{0} {1} {2} -p {3} {4} {5}@{6}".format(
                "ssh-copy-id",
                "-i {0}.pub".format(self.priv),
                self._passwd_opts(),
                self.port,
                self._ssh_opts(),
                self.user,
                self.host,
            )
        return None

    def copy_id(self):
        """
        Execute ssh-copy-id to plant the id file on the target
        """
        stdout, stderr, retcode = self._run_cmd(self._copy_id_str_old())
        if salt.defaults.exitcodes.EX_OK != retcode and "Usage" in stderr:
            stdout, stderr, retcode = self._run_cmd(self._copy_id_str_new())
        return stdout, stderr, retcode

    def _cmd_str(self, cmd, ssh="ssh"):
        """
        Return the cmd string to execute
        """

        # TODO: if tty, then our SSH_SHIM cannot be supplied from STDIN Will
        # need to deliver the SHIM to the remote host and execute it there

        command = [ssh]
        if ssh != "scp":
            command.append(self.host)
        if self.tty and ssh == "ssh":
            command.append("-t -t")
        if self.passwd or self.priv:
            command.append(self.priv and self._key_opts() or self._passwd_opts())
        if ssh != "scp" and self.remote_port_forwards:
            command.append(
                " ".join(
                    [
                        "-R {0}".format(item)
                        for item in self.remote_port_forwards.split(",")
                    ]
                )
            )
        if self.ssh_options:
            command.append(self._ssh_opts())

        command.append(cmd)

        return " ".join(command)

    def _old_run_cmd(self, cmd):
        """
        Cleanly execute the command string
        """
        try:
            proc = subprocess.Popen(
                cmd, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE,
            )

            data = proc.communicate()
            return data[0], data[1], proc.returncode
        except Exception:  # pylint: disable=broad-except
            return ("local", "Unknown Error", None)

    def _run_nb_cmd(self, cmd):
        """
        cmd iterator
        """
        try:
            proc = salt.utils.nb_popen.NonBlockingPopen(
                cmd, shell=True, stderr=subprocess.PIPE, stdout=subprocess.PIPE,
            )
            while True:
                time.sleep(0.1)
                out = proc.recv()
                err = proc.recv_err()
                rcode = proc.returncode
                if out is None and err is None:
                    break
                if err:
                    err = self.get_error(err)
                yield out, err, rcode
        except Exception:  # pylint: disable=broad-except
            yield ("", "Unknown Error", None)

    def exec_nb_cmd(self, cmd):
        """
        Yield None until cmd finished
        """
        r_out = []
        r_err = []
        rcode = None
        cmd = self._cmd_str(cmd)

        logmsg = "Executing non-blocking command: {0}".format(cmd)
        if self.passwd:
            logmsg = logmsg.replace(self.passwd, ("*" * 6))
        log.debug(logmsg)

        for out, err, rcode in self._run_nb_cmd(cmd):
            if out is not None:
                r_out.append(out)
            if err is not None:
                r_err.append(err)
            yield None, None, None
        yield "".join(r_out), "".join(r_err), rcode

    def exec_cmd(self, cmd):
        """
        Execute a remote command
        """
        cmd = self._cmd_str(cmd)

        logmsg = "Executing command: {0}".format(cmd)
        if self.passwd:
            logmsg = logmsg.replace(self.passwd, ("*" * 6))
        if 'decode("base64")' in logmsg or "base64.b64decode(" in logmsg:
            log.debug("Executed SHIM command. Command logged to TRACE")
            log.trace(logmsg)
        else:
            log.debug(logmsg)

        ret = self._run_cmd(cmd)
        return ret

    def send(self, local, remote, makedirs=False):
        """
        scp a file or files to a remote system
        """
        if makedirs:
            self.exec_cmd("mkdir -p {0}".format(os.path.dirname(remote)))

        # scp needs [<ipv6}
        host = self.host
        if ":" in host:
            host = "[{0}]".format(host)

        cmd = "{0} {1}:{2}".format(local, host, remote)
        cmd = self._cmd_str(cmd, ssh="scp")

        logmsg = "Executing command: {0}".format(cmd)
        if self.passwd:
            logmsg = logmsg.replace(self.passwd, ("*" * 6))
        log.debug(logmsg)

        return self._run_cmd(cmd)

    def _run_cmd(self, cmd, key_accept=False, passwd_retries=3):
        """
        Execute a shell command via VT. This is blocking and assumes that ssh
        is being run
        """
        if not cmd:
            return "", "No command or passphrase", 245

        term = salt.utils.vt.Terminal(
            cmd,
            shell=True,
            log_stdout=True,
            log_stdout_level="trace",
            log_stderr=True,
            log_stderr_level="trace",
            stream_stdout=False,
            stream_stderr=False,
        )
        sent_passwd = 0
        send_password = True
        ret_stdout = ""
        ret_stderr = ""
        old_stdout = ""

        try:
            while term.has_unread_data:
                stdout, stderr = term.recv()
                if stdout:
                    ret_stdout += stdout
                    buff = old_stdout + stdout
                else:
                    buff = stdout
                if stderr:
                    ret_stderr += stderr
                if buff and RSTR_RE.search(buff):
                    # We're getting results back, don't try to send passwords
                    send_password = False
                if buff and SSH_PRIVATE_KEY_PASSWORD_PROMPT_RE.search(buff):
                    if not self.priv_passwd:
                        return "", "Private key file need passphrase", 254
                    term.sendline(self.priv_passwd)
                    continue
                if buff and SSH_PASSWORD_PROMPT_RE.search(buff) and send_password:
                    if not self.passwd:
                        return (
                            "",
                            "Permission denied, no authentication information",
                            254,
                        )
                    if sent_passwd < passwd_retries:
                        term.sendline(self.passwd)
                        sent_passwd += 1
                        continue
                    else:
                        # asking for a password, and we can't seem to send it
                        return "", "Password authentication failed", 254
                elif buff and KEY_VALID_RE.search(buff):
                    if key_accept:
                        term.sendline("yes")
                        continue
                    else:
                        term.sendline("no")
                        ret_stdout = (
                            "The host key needs to be accepted, to "
                            "auto accept run salt-ssh with the -i "
                            "flag:\n{0}"
                        ).format(stdout)
                        return ret_stdout, "", 254
                elif buff and buff.endswith("_||ext_mods||_"):
                    mods_raw = (
                        salt.utils.json.dumps(self.mods, separators=(",", ":"))
                        + "|_E|0|"
                    )
                    term.sendline(mods_raw)
                if stdout:
                    old_stdout = stdout
                time.sleep(0.01)
            return ret_stdout, ret_stderr, term.exitstatus
        finally:
            term.close(terminate=True, kill=True)



# -*- coding: utf-8 -*-
"""
unit tests for the script engine
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.config
import salt.engines.script as script
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class EngineScriptTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.engine.script
    """

    def setup_loader_modules(self):

        opts = salt.config.DEFAULT_MASTER_OPTS
        return {script: {"__opts__": opts}}

    def test__get_serializer(self):
        """
        Test known serializer is returned or exception is raised
        if unknown serializer
        """
        for serializers in ("json", "yaml", "msgpack"):
            self.assertTrue(script._get_serializer(serializers))

        with self.assertRaises(CommandExecutionError):
            script._get_serializer("bad")

    def test__read_stdout(self):
        """
        Test we can yield stdout
        """
        with patch("subprocess.Popen") as popen_mock:
            popen_mock.stdout.readline.return_value = "test"
            self.assertEqual(next(script._read_stdout(popen_mock)), "test")



# -*- coding: utf-8 -*-
"""
    :synopsis: Unit Tests for Package Management module 'module.opkg'
    :platform: Linux
"""
from __future__ import absolute_import, print_function, unicode_literals

import collections
import copy

import salt.modules.opkg as opkg
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class OpkgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.opkg
    """

    @classmethod
    def setUpClass(cls):
        cls.opkg_vim_info = {
            "vim": {
                "Package": "vim",
                "Version": "7.4.769-r0.31",
                "Status": "install ok installed",
            }
        }
        cls.opkg_vim_files = {
            "errors": [],
            "packages": {
                "vim": [
                    "/usr/bin/view",
                    "/usr/bin/vim.vim",
                    "/usr/bin/xxd",
                    "/usr/bin/vimdiff",
                    "/usr/bin/rview",
                    "/usr/bin/rvim",
                    "/usr/bin/ex",
                ]
            },
        }
        cls.installed = {"vim": {"new": "7.4", "old": ""}}
        cls.removed = {"vim": {"new": "", "old": "7.4"}}
        cls.packages = {"vim": "7.4"}

    @classmethod
    def tearDownClass(cls):
        cls.opkg_vim_info = (
            cls.opkg_vim_files
        ) = cls.installed = cls.removed = cls.packages = None

    def setup_loader_modules(self):  # pylint: disable=no-self-use
        """
        Tested modules
        """
        return {opkg: {}}

    def test_version(self):
        """
        Test - Returns a string representing the package version or an empty string if
        not installed.
        """
        version = self.opkg_vim_info["vim"]["Version"]
        mock = MagicMock(return_value=version)
        with patch.dict(opkg.__salt__, {"pkg_resource.version": mock}):
            self.assertEqual(opkg.version(*["vim"]), version)

    def test_upgrade_available(self):
        """
        Test - Check whether or not an upgrade is available for a given package.
        """
        with patch("salt.modules.opkg.latest_version", MagicMock(return_value="")):
            self.assertFalse(opkg.upgrade_available("vim"))

    def test_file_dict(self):
        """
        Test - List the files that belong to a package, grouped by package.
        """
        std_out = "\n".join(self.opkg_vim_files["packages"]["vim"])
        ret_value = {"stdout": std_out}
        mock = MagicMock(return_value=ret_value)
        with patch.dict(opkg.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(opkg.file_dict("vim"), self.opkg_vim_files)

    def test_file_list(self):
        """
        Test - List the files that belong to a package.
        """
        std_out = "\n".join(self.opkg_vim_files["packages"]["vim"])
        ret_value = {"stdout": std_out}
        mock = MagicMock(return_value=ret_value)
        files = {
            "errors": self.opkg_vim_files["errors"],
            "files": self.opkg_vim_files["packages"]["vim"],
        }
        with patch.dict(opkg.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(opkg.file_list("vim"), files)

    def test_owner(self):
        """
        Test - Return the name of the package that owns the file.
        """
        paths = ["/usr/bin/vimdiff"]
        mock = MagicMock(return_value="vim - version - info")
        with patch.dict(opkg.__salt__, {"cmd.run_stdout": mock}):
            self.assertEqual(opkg.owner(*paths), "vim")

    def test_install(self):
        """
        Test - Install packages.
        """
        with patch(
            "salt.modules.opkg.list_pkgs", MagicMock(side_effect=[{}, self.packages])
        ):
            ret_value = {"retcode": 0}
            mock = MagicMock(return_value=ret_value)
            patch_kwargs = {
                "__salt__": {
                    "cmd.run_all": mock,
                    "pkg_resource.parse_targets": MagicMock(
                        return_value=({"vim": "7.4"}, "repository")
                    ),
                    "restartcheck.restartcheck": MagicMock(
                        return_value="No packages seem to need to be restarted"
                    ),
                }
            }
            with patch.multiple(opkg, **patch_kwargs):
                self.assertEqual(opkg.install("vim:7.4"), self.installed)

    def test_install_noaction(self):
        """
        Test - Install packages.
        """
        with patch("salt.modules.opkg.list_pkgs", MagicMock(side_effect=({}, {}))):
            std_out = "Downloading http://feedserver/feeds/test/vim_7.4_arch.ipk.\n\nInstalling vim (7.4) on root\n"
            ret_value = {"retcode": 0, "stdout": std_out}
            mock = MagicMock(return_value=ret_value)
            patch_kwargs = {
                "__salt__": {
                    "cmd.run_all": mock,
                    "pkg_resource.parse_targets": MagicMock(
                        return_value=({"vim": "7.4"}, "repository")
                    ),
                    "restartcheck.restartcheck": MagicMock(
                        return_value="No packages seem to need to be restarted"
                    ),
                }
            }
            with patch.multiple(opkg, **patch_kwargs):
                self.assertEqual(opkg.install("vim:7.4", test=True), self.installed)

    def test_remove(self):
        """
        Test - Remove packages.
        """
        with patch(
            "salt.modules.opkg.list_pkgs", MagicMock(side_effect=[self.packages, {}])
        ):
            ret_value = {"retcode": 0}
            mock = MagicMock(return_value=ret_value)
            patch_kwargs = {
                "__salt__": {
                    "cmd.run_all": mock,
                    "pkg_resource.parse_targets": MagicMock(
                        return_value=({"vim": "7.4"}, "repository")
                    ),
                    "restartcheck.restartcheck": MagicMock(
                        return_value="No packages seem to need to be restarted"
                    ),
                }
            }
            with patch.multiple(opkg, **patch_kwargs):
                self.assertEqual(opkg.remove("vim"), self.removed)

    def test_remove_noaction(self):
        """
        Test - Remove packages.
        """
        with patch(
            "salt.modules.opkg.list_pkgs",
            MagicMock(side_effect=[self.packages, self.packages]),
        ):
            std_out = "\nRemoving vim (7.4) from root...\n"
            ret_value = {"retcode": 0, "stdout": std_out}
            mock = MagicMock(return_value=ret_value)
            patch_kwargs = {
                "__salt__": {
                    "cmd.run_all": mock,
                    "pkg_resource.parse_targets": MagicMock(
                        return_value=({"vim": "7.4"}, "repository")
                    ),
                    "restartcheck.restartcheck": MagicMock(
                        return_value="No packages seem to need to be restarted"
                    ),
                }
            }
            with patch.multiple(opkg, **patch_kwargs):
                self.assertEqual(opkg.remove("vim:7.4", test=True), self.removed)

    def test_info_installed(self):
        """
        Test - Return the information of the named package(s) installed on the system.
        """
        installed = copy.deepcopy(self.opkg_vim_info["vim"])
        del installed["Package"]
        ordered_info = collections.OrderedDict(sorted(installed.items()))
        expected_dict = {"vim": {k.lower(): v for k, v in ordered_info.items()}}
        std_out = "\n".join(
            [k + ": " + v for k, v in self.opkg_vim_info["vim"].items()]
        )
        ret_value = {"stdout": std_out, "retcode": 0}
        mock = MagicMock(return_value=ret_value)
        with patch.dict(opkg.__salt__, {"cmd.run_all": mock}):
            self.assertEqual(opkg.info_installed("vim"), expected_dict)

    def test_version_clean(self):
        """
        Test - Return the information of version_clean
        """
        self.assertEqual(opkg.version_clean("1.0.1"), "1.0.1")

    def test_check_extra_requirements(self):
        """
        Test - Return the information of check_extra_requirements
        """
        self.assertEqual(opkg.check_extra_requirements("vim", "1.0.1"), True)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import random
import string

import salt.loader
import salt.states.boto_elasticsearch_domain as boto_elasticsearch_domain

# Import Salt libs
from salt.ext import six

# Import 3rd-party libs
from salt.ext.six.moves import range  # pylint: disable=import-error,redefined-builtin
from salt.utils.versions import LooseVersion

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

# pylint: disable=import-error,no-name-in-module,unused-import
from tests.unit.modules.test_boto_elasticsearch_domain import (
    BotoElasticsearchDomainTestCaseMixin,
)

# Import test suite libs


try:
    import boto
    import boto3
    from botocore.exceptions import ClientError

    HAS_BOTO = True
except ImportError:
    HAS_BOTO = False

# pylint: enable=import-error,no-name-in-module,unused-import

# the boto_elasticsearch_domain module relies on the connect_to_region() method
# which was added in boto 2.8.0
# https://github.com/boto/boto/commit/33ac26b416fbb48a60602542b4ce15dcc7029f12
required_boto3_version = "1.2.1"

log = logging.getLogger(__name__)


def _has_required_boto():
    """
    Returns True/False boolean depending on if Boto is installed and correct
    version.
    """
    if not HAS_BOTO:
        return False
    elif LooseVersion(boto3.__version__) < LooseVersion(required_boto3_version):
        return False
    else:
        return True


if _has_required_boto():
    region = "us-east-1"
    access_key = "GKTADJGHEIQSXMKKRBJ08H"
    secret_key = "askdjghsdfjkghWupUjasdflkdfklgjsdfjajkghs"
    conn_parameters = {
        "region": region,
        "key": access_key,
        "keyid": secret_key,
        "profile": {},
    }
    error_message = (
        "An error occurred (101) when calling the {0} operation: Test-defined error"
    )
    not_found_error = ClientError(
        {
            "Error": {
                "Code": "ResourceNotFoundException",
                "Message": "Test-defined error",
            }
        },
        "msg",
    )
    error_content = {"Error": {"Code": 101, "Message": "Test-defined error"}}
    domain_ret = dict(
        DomainName="testdomain",
        ElasticsearchClusterConfig={},
        EBSOptions={},
        AccessPolicies={},
        SnapshotOptions={},
        AdvancedOptions={},
        ElasticsearchVersion="1.5",
    )


class BotoElasticsearchDomainStateTestCaseBase(TestCase, LoaderModuleMockMixin):
    conn = None

    def setup_loader_modules(self):
        ctx = {}
        utils = salt.loader.utils(
            self.opts,
            whitelist=["boto3", "args", "systemd", "path", "platform"],
            context=ctx,
        )
        serializers = salt.loader.serializers(self.opts)
        self.funcs = funcs = salt.loader.minion_mods(
            self.opts, context=ctx, utils=utils, whitelist=["boto_elasticsearch_domain"]
        )
        self.salt_states = salt.loader.states(
            opts=self.opts,
            functions=funcs,
            utils=utils,
            whitelist=["boto_elasticsearch_domain"],
            serializers=serializers,
        )
        return {
            boto_elasticsearch_domain: {
                "__opts__": self.opts,
                "__salt__": funcs,
                "__utils__": utils,
                "__states__": self.salt_states,
                "__serializers__": serializers,
            }
        }

    @classmethod
    def setUpClass(cls):
        cls.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        cls.opts["grains"] = salt.loader.grains(cls.opts)

    @classmethod
    def tearDownClass(cls):
        del cls.opts

    # Set up MagicMock to replace the boto3 session
    def setUp(self):
        self.addCleanup(delattr, self, "funcs")
        self.addCleanup(delattr, self, "salt_states")
        # Set up MagicMock to replace the boto3 session
        # connections keep getting cached from prior tests, can't find the
        # correct context object to clear it. So randomize the cache key, to prevent any
        # cache hits
        conn_parameters["key"] = "".join(
            random.choice(string.ascii_lowercase + string.digits) for _ in range(50)
        )

        self.patcher = patch("boto3.session.Session")
        self.addCleanup(self.patcher.stop)
        self.addCleanup(delattr, self, "patcher")
        mock_session = self.patcher.start()

        session_instance = mock_session.return_value
        self.conn = MagicMock()
        self.addCleanup(delattr, self, "conn")
        session_instance.client.return_value = self.conn


@skipIf(HAS_BOTO is False, "The boto module must be installed.")
@skipIf(
    _has_required_boto() is False,
    "The boto3 module must be greater than"
    " or equal to version {0}".format(required_boto3_version),
)
class BotoElasticsearchDomainTestCase(
    BotoElasticsearchDomainStateTestCaseBase, BotoElasticsearchDomainTestCaseMixin
):
    """
    TestCase for salt.modules.boto_elasticsearch_domain state.module
    """

    def test_present_when_domain_does_not_exist(self):
        """
        Tests present on a domain that does not exist.
        """
        self.conn.describe_elasticsearch_domain.side_effect = not_found_error
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": domain_ret
        }
        self.conn.create_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        result = self.salt_states["boto_elasticsearch_domain.present"](
            "domain present", **domain_ret
        )

        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"]["new"]["domain"]["ElasticsearchClusterConfig"], None
        )

    def test_present_when_domain_exists(self):
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        cfg = {}
        for k, v in six.iteritems(domain_ret):
            cfg[k] = {"Options": v}
        cfg["AccessPolicies"] = {"Options": '{"a": "b"}'}
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": cfg
        }
        self.conn.update_elasticsearch_domain_config.return_value = {
            "DomainConfig": cfg
        }
        result = self.salt_states["boto_elasticsearch_domain.present"](
            "domain present", **domain_ret
        )
        self.assertTrue(result["result"])
        self.assertEqual(
            result["changes"],
            {"new": {"AccessPolicies": {}}, "old": {"AccessPolicies": {"a": "b"}}},
        )

    def test_present_with_failure(self):
        self.conn.describe_elasticsearch_domain.side_effect = not_found_error
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": domain_ret
        }
        self.conn.create_elasticsearch_domain.side_effect = ClientError(
            error_content, "create_domain"
        )
        result = self.salt_states["boto_elasticsearch_domain.present"](
            "domain present", **domain_ret
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])

    def test_absent_when_domain_does_not_exist(self):
        """
        Tests absent on a domain that does not exist.
        """
        self.conn.describe_elasticsearch_domain.side_effect = not_found_error
        result = self.salt_states["boto_elasticsearch_domain.absent"](
            "test", "mydomain"
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"], {})

    def test_absent_when_domain_exists(self):
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": domain_ret
        }
        result = self.salt_states["boto_elasticsearch_domain.absent"](
            "test", domain_ret["DomainName"]
        )
        self.assertTrue(result["result"])
        self.assertEqual(result["changes"]["new"]["domain"], None)

    def test_absent_with_failure(self):
        self.conn.describe_elasticsearch_domain.return_value = {
            "DomainStatus": domain_ret
        }
        self.conn.describe_elasticsearch_domain_config.return_value = {
            "DomainConfig": domain_ret
        }
        self.conn.delete_elasticsearch_domain.side_effect = ClientError(
            error_content, "delete_domain"
        )
        result = self.salt_states["boto_elasticsearch_domain.absent"](
            "test", domain_ret["DomainName"]
        )
        self.assertFalse(result["result"])
        self.assertTrue("An error occurred" in result["comment"])



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import tests.support.napalm as napalm_test_support

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase

import salt.modules.napalm_route as napalm_route  # NOQA


def mock_net_load(template, *args, **kwargs):
    raise ValueError("incorrect template {0}".format(template))


class NapalmRouteModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        module_globals = {
            "__salt__": {
                "config.option": MagicMock(
                    return_value={"test": {"driver": "test", "key": "2orgk34kgk34g"}}
                ),
                "file.file_exists": napalm_test_support.true,
                "file.join": napalm_test_support.join,
                "file.get_managed": napalm_test_support.get_managed_file,
                "random.hash": napalm_test_support.random_hash,
                "net.load_template": mock_net_load,
            }
        }

        return {napalm_route: module_globals}

    def test_show(self):
        ret = napalm_route.show("1.2.3.4")
        assert ret["out"] == napalm_test_support.TEST_ROUTE



# -*- coding: utf-8 -*-
"""
Tests for minion blackout
"""

from __future__ import absolute_import

import logging
import os
import textwrap
import time

import pytest
import salt.utils.files
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class MinionBlackoutTestCase(ModuleCase):
    """
    Test minion blackout functionality
    """

    @classmethod
    def setUpClass(cls):
        cls.top_pillar = os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "top.sls")
        cls.blackout_pillar = os.path.join(RUNTIME_VARS.TMP_PILLAR_TREE, "blackout.sls")

    @classmethod
    def tearDownClass(cls):
        if os.path.exists(cls.top_pillar):
            os.unlink(cls.top_pillar)
        del cls.top_pillar
        if os.path.exists(cls.blackout_pillar):
            os.unlink(cls.blackout_pillar)
        del cls.blackout_pillar

    def setUp(self):
        with salt.utils.files.fopen(self.top_pillar, "w") as wfh:
            wfh.write(
                textwrap.dedent(
                    """\
                base:
                  '*':
                    - blackout
                """
                )
            )
        with salt.utils.files.fopen(self.blackout_pillar, "w") as wfh:
            wfh.write("minion_blackout: False")
        self.addCleanup(self.cleanup_blackout_pillar)

    def tearDown(self):
        self.end_blackout()
        # Be sure to also refresh the sub_minion pillar
        self.run_function("saltutil.refresh_pillar", minion_tgt="sub_minion")
        timeout = 120
        if not self.wait_for_blackout(end=True, tgt="sub_minion", timeout=timeout):
            raise Exception(
                "Minion did not exit blackout mode after {} seconds".format(timeout)
            )
        self.wait_for_all_jobs()

    def cleanup_blackout_pillar(self):
        if os.path.exists(self.top_pillar):
            os.unlink(self.top_pillar)
        if os.path.exists(self.blackout_pillar):
            os.unlink(self.blackout_pillar)

    def begin_blackout(self, blackout_data="minion_blackout: True"):
        """
        setup minion blackout mode
        """
        log.info("Entering minion blackout...")
        self.wait_for_all_jobs()
        with salt.utils.files.fopen(self.blackout_pillar, "w") as wfh:
            wfh.write(blackout_data)
        ret = self.run_function("saltutil.refresh_pillar", timeout=30)
        timeout = 120
        if not self.wait_for_blackout(timeout=timeout):
            raise Exception(
                "Minion did not enter blackout mode after {} seconds".format(timeout)
            )
        log.info("Entered minion blackout.")

    def wait_for_blackout(self, end=False, tgt="minion", timeout=120, sleep=0.3):
        """
        Wait for blackout mode to start or end.
        """
        start = time.time()
        while time.time() - start <= timeout:
            ret = self.run_function(
                "pillar.get", minion_tgt=tgt, arg=["minion_blackout"], timeout=30,
            )
            if end:
                if str(ret).find("Minion in blackout mode") == -1:
                    return True
            else:
                if str(ret).find("Minion in blackout mode") != -1:
                    return True
            time.sleep(sleep)
        return False

    def end_blackout(self):
        """
        takedown minion blackout mode
        """
        log.info("Exiting minion blackout...")
        with salt.utils.files.fopen(self.blackout_pillar, "w") as wfh:
            wfh.write("minion_blackout: False\n")
        self.run_function("saltutil.refresh_pillar")
        timeout = 120
        if not self.wait_for_blackout(end=True, timeout=timeout):
            raise Exception(
                "Minion did not exit blackout mode after {} seconds".format(timeout)
            )
        self.wait_for_all_jobs()
        log.info("Exited minion blackout.")

    @slowTest
    def test_blackout(self):
        """
        Test that basic minion blackout functionality works
        """
        try:
            self.begin_blackout()
            blackout_ret = self.run_function("test.ping")
            self.assertIn("Minion in blackout mode.", blackout_ret)
        finally:
            self.end_blackout()

        ret = self.run_function("test.ping")
        self.assertEqual(ret, True)

    @slowTest
    def test_blackout_whitelist(self):
        """
        Test that minion blackout whitelist works
        """
        self.begin_blackout(
            textwrap.dedent(
                """\
            minion_blackout: True
            minion_blackout_whitelist:
              - test.ping
              - test.fib
            """
            )
        )

        ping_ret = self.run_function("test.ping")
        self.assertEqual(ping_ret, True)

        fib_ret = self.run_function("test.fib", [7])
        self.assertTrue(isinstance(fib_ret, list))
        self.assertEqual(fib_ret[0], 13)

    @slowTest
    def test_blackout_nonwhitelist(self):
        """
        Test that minion refuses to run non-whitelisted functions during
        blackout whitelist
        """
        self.begin_blackout(
            textwrap.dedent(
                """\
            minion_blackout: True
            minion_blackout_whitelist:
              - test.ping
              - test.fib
            """
            )
        )

        state_ret = self.run_function("state.apply")
        self.assertIn("Minion in blackout mode.", state_ret)

        cloud_ret = self.run_function("cloud.query", ["list_nodes_full"])
        self.assertIn("Minion in blackout mode.", cloud_ret)



# -*- coding: utf-8 -*-
"""
unit tests for the beacon_module parameter
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import Salt Libs
import salt.beacons as beacons
import salt.config

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class BeaconsTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt beacon_module parameter
    """

    def setup_loader_modules(self):
        return {beacons: {}}

    def test_beacon_module(self):
        """
        Test that beacon_module parameter for beacon configuration
        """
        mock_opts = salt.config.DEFAULT_MINION_OPTS.copy()
        mock_opts["id"] = "minion"
        mock_opts["__role"] = "minion"
        mock_opts["beacons"] = {
            "watch_apache": [
                {"processes": {"apache2": "stopped"}},
                {"beacon_module": "ps"},
            ]
        }
        with patch.dict(beacons.__opts__, mock_opts):
            ret = salt.beacons.Beacon(mock_opts, []).process(
                mock_opts["beacons"], mock_opts["grains"]
            )
            _expected = [
                {
                    "tag": "salt/beacon/minion/watch_apache/",
                    "data": {"id": "minion", "apache2": "Stopped"},
                    "beacon_name": "ps",
                }
            ]
            self.assertEqual(ret, _expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import

# Import Salt Libs
import salt.states.pkgng as pkgng

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PkgngTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.pkgng
    """

    def setup_loader_modules(self):
        return {pkgng: {}}

    # 'update_packaging_site' function tests: 1

    def test_update_packaging_site(self):
        """
        Test to execute update_packaging_site.
        """
        name = "http://192.168.0.2"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock_t = MagicMock(return_value=True)
        with patch.dict(pkgng.__salt__, {"pkgng.update_package_site": mock_t}):
            self.assertDictEqual(pkgng.update_packaging_site(name), ret)



# -*- coding: utf-8 -*-
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.apache_site as apache_site

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ApacheSiteTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.apache_site
    """

    def setup_loader_modules(self):
        return {apache_site: {}}

    # 'enabled' function tests: 1

    def test_enabled(self):
        """
        Test to ensure an Apache site is enabled.
        """
        name = "saltstack.com"

        ret = {"name": name, "result": True, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, False, False])
        mock_str = MagicMock(return_value={"Status": ["enabled"]})
        with patch.dict(
            apache_site.__salt__,
            {"apache.check_site_enabled": mock, "apache.a2ensite": mock_str},
        ):
            comt = "{0} already enabled.".format(name)
            ret.update({"comment": comt})
            self.assertDictEqual(apache_site.enabled(name), ret)

            comt = "Apache site {0} is set to be enabled.".format(name)
            ret.update(
                {"comment": comt, "result": None, "changes": {"new": name, "old": None}}
            )
            with patch.dict(apache_site.__opts__, {"test": True}):
                self.assertDictEqual(apache_site.enabled(name), ret)

            comt = "Failed to enable {0} Apache site".format(name)
            ret.update({"comment": comt, "result": False, "changes": {}})
            with patch.dict(apache_site.__opts__, {"test": False}):
                self.assertDictEqual(apache_site.enabled(name), ret)

    # 'disabled' function tests: 1

    def test_disabled(self):
        """
        Test to ensure an Apache site is disabled.
        """
        name = "saltstack.com"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, True, False])
        mock_str = MagicMock(return_value={"Status": ["disabled"]})
        with patch.dict(
            apache_site.__salt__,
            {"apache.check_site_enabled": mock, "apache.a2dissite": mock_str},
        ):
            comt = "Apache site {0} is set to be disabled.".format(name)
            ret.update({"comment": comt, "changes": {"new": None, "old": name}})
            with patch.dict(apache_site.__opts__, {"test": True}):
                self.assertDictEqual(apache_site.disabled(name), ret)

            comt = "Failed to disable {0} Apache site".format(name)
            ret.update({"comment": comt, "result": False, "changes": {}})
            with patch.dict(apache_site.__opts__, {"test": False}):
                self.assertDictEqual(apache_site.disabled(name), ret)

            comt = "{0} already disabled.".format(name)
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(apache_site.disabled(name), ret)



# -*- coding: utf-8 -*-
"""
Test the win_wua state module
"""
# Import Python Libs
from __future__ import (  # Import Salt Libs
    absolute_import,
    print_function,
    unicode_literals,
)

import salt.states.win_wua as win_wua
import salt.utils.platform
import salt.utils.win_update as win_update
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf

UPDATES_LIST = {
    "ca3bb521-a8ea-4e26-a563-2ad6e3108b9a": {
        "KBs": ["KB4481252"],
        "Installed": False,
        "Title": "Blank",
    },
    "07609d43-d518-4e77-856e-d1b316d1b8a8": {
        "KBs": ["KB925673"],
        "Installed": False,
        "Title": "Blank",
    },
    "fbaa5360-a440-49d8-a3b6-0c4fc7ecaa19": {
        "KBs": ["KB4481252"],
        "Installed": False,
        "Title": "Blank",
    },
    "a873372b-7a5c-443c-8022-cd59a550bef4": {
        "KBs": ["KB3193497"],
        "Installed": False,
        "Title": "Blank",
    },
    "14075cbe-822e-4004-963b-f50e08d45563": {
        "KBs": ["KB4540723"],
        "Installed": False,
        "Title": "Blank",
    },
    "d931e99c-4dda-4d39-9905-0f6a73f7195f": {
        "KBs": ["KB3193497"],
        "Installed": False,
        "Title": "Blank",
    },
    "afda9e11-44a0-4602-9e9b-423af11ecaed": {
        "KBs": ["KB4541329"],
        "Installed": False,
        "Title": "Blank",
    },
    "a0f997b1-1abe-4a46-941f-b37f732f9fbd": {
        "KBs": ["KB3193497"],
        "Installed": False,
        "Title": "Blank",
    },
    "eac02b09-d745-4891-b80f-400e0e5e4b6d": {
        "KBs": ["KB4052623"],
        "Installed": False,
        "Title": "Blank",
    },
    "0689e74b-54d1-4f55-a916-96e3c737db90": {
        "KBs": ["KB890830"],
        "Installed": False,
        "Title": "Blank",
    },
}


UPDATES_LIST_NONE = {}


UPDATES_SUMMARY = {"Installed": 10}


class Updates(object):
    def __init__(self):
        self.updates = []


@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class WinWuaTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test the functions in the win_wua.uptodate function
    """

    def setup_loader_modules(self):
        return {win_wua: {"__opts__": {"test": False}, "__env__": "base"}}

    def test_uptodate_no_updates(self):
        """
        Test uptodate function with no updates found.
        """
        expected = {
            "name": "NA",
            "changes": {},
            "result": True,
            "comment": "No updates found",
        }

        class NoUpdates(Updates):
            @staticmethod
            def updates():  # pylint: disable=method-hidden
                return UPDATES_LIST_NONE

            @staticmethod
            def count():
                return len(UPDATES_LIST_NONE)

        patch_winapi_com = patch("salt.utils.winapi.Com", autospec=True)
        patch_win32com = patch("win32com.client.Dispatch", autospec=True)
        patch_win_update_agent = patch.object(
            salt.utils.win_update.WindowsUpdateAgent, "refresh", autospec=True
        )
        patch_win_update = patch.object(
            salt.utils.win_update, "Updates", autospec=True, return_value=NoUpdates()
        )

        with patch_winapi_com, patch_win32com, patch_win_update_agent, patch_win_update:
            result = win_wua.uptodate(name="NA")
            self.assertDictEqual(result, expected)

    def test_uptodate_testmode(self):
        """
        Test uptodate function in test=true mode.
        """
        expected = {
            "name": "NA",
            "changes": {},
            "result": None,
            "comment": "Updates will be installed:",
        }
        patch_winapi_com = patch("salt.utils.winapi.Com", autospec=True)
        patch_win32com = patch("win32com.client.Dispatch", autospec=True)
        patch_win_update_agent = patch.object(
            salt.utils.win_update.WindowsUpdateAgent, "refresh", autospec=True
        )
        patch_opts = patch.dict(win_wua.__opts__, {"test": True})

        with patch_winapi_com, patch_win32com, patch_win_update_agent, patch_opts:
            wua = win_update.WindowsUpdateAgent(online=False)
            wua._updates = [MagicMock(IsInstalled=False, IsDownloaded=False)]
            result = win_wua.uptodate(name="NA")
            self.assertDictEqual(result, expected)

    def test_uptodate(self):
        """
        Test uptodate function with some updates found.
        """
        expected = {
            "name": "NA",
            "changes": {
                "failed": {
                    "eac02b09-d745-4891-b80f-400e0e5e4b6d": {
                        "Title": "Blank...",
                        "KBs": ["KB4052623"],
                    }
                }
            },
            "result": False,
            "comment": "Updates failed",
        }

        updates_not_installed = {
            "afda9e11-44a0-4602-9e9b-423af11ecaed": {
                "KBs": ["KB4541329"],
                "Installed": False,
                "Title": "Blank",
            },
            "a0f997b1-1abe-4a46-941f-b37f732f9fbd": {
                "KBs": ["KB3193497"],
                "Installed": False,
                "Title": "Blank",
            },
            "eac02b09-d745-4891-b80f-400e0e5e4b6d": {
                "KBs": ["KB4052623"],
                "Installed": False,
                "Title": "Blank",
            },
            "eac02c07-d744-4892-b80f-312d045e4ccc": {
                "KBs": ["KB4052444"],
                "Installed": False,
                "Title": "Blank",
            },
        }
        fake_wua = MagicMock()
        fake_updates = MagicMock()
        fake_updates.list.return_value = updates_not_installed

        fake_wua_updates = MagicMock()
        fake_wua_updates.list.return_value = UPDATES_LIST
        fake_wua.updates.return_value = fake_wua_updates

        patch_winapi_com = patch("salt.utils.winapi.Com", autospec=True)
        patch_win32 = patch("win32com.client.Dispatch", autospec=True)
        patch_wua = patch(
            "salt.utils.win_update.WindowsUpdateAgent",
            autospec=True,
            return_value=fake_wua,
        )
        patch_win_wua_update = patch(
            "salt.utils.win_update.Updates", autospec=True, return_value=fake_updates
        )
        patch_opts = patch.dict(win_wua.__opts__, {"test": False})

        with patch_winapi_com, patch_win32, patch_wua, patch_win_wua_update, patch_opts:
            wua = win_update.WindowsUpdateAgent(online=False)
            result = win_wua.uptodate(name="NA")
            self.assertDictEqual(result, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Eric Radman <ericshane@eradman.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.openbsdpkg as openbsdpkg

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, call, patch
from tests.support.unit import TestCase


class ListPackages(object):
    def __init__(self):
        self._iteration = 0

    def __call__(self):
        pkg_lists = [
            {"vim": "7.4.1467p1-gtk2"},
            {"png": "1.6.23", "vim": "7.4.1467p1-gtk2", "ruby": "2.3.1p1"},
        ]
        pkgs = pkg_lists[self._iteration]
        self._iteration += 1
        return pkgs


class OpenbsdpkgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.openbsdpkg
    """

    def setup_loader_modules(self):
        return {openbsdpkg: {}}

    def test_list_pkgs(self):
        """
        Test for listing installed packages.
        """

        def _add_data(data, key, value):
            data[key] = value

        pkg_info_out = [
            "png-1.6.23",
            "vim-7.4.1467p1-gtk2",  # vim--gtk2
            "ruby-2.3.1p1",  # ruby%2.3
        ]
        run_stdout_mock = MagicMock(return_value="\n".join(pkg_info_out))
        patches = {
            "cmd.run_stdout": run_stdout_mock,
            "pkg_resource.add_pkg": _add_data,
            "pkg_resource.sort_pkglist": MagicMock(),
            "pkg_resource.stringify": MagicMock(),
        }
        with patch.dict(openbsdpkg.__salt__, patches):
            pkgs = openbsdpkg.list_pkgs()
            self.assertDictEqual(
                pkgs, {"png": "1.6.23", "vim--gtk2": "7.4.1467p1", "ruby": "2.3.1p1"}
            )
        run_stdout_mock.assert_called_once_with(
            "pkg_info -q -a", output_loglevel="trace"
        )

    def test_install_pkgs(self):
        """
        Test package install behavior for the following conditions:
        - only base package name is given ('png')
        - a flavor is specified ('vim--gtk2')
        - a branch is specified ('ruby%2.3')
        """

        parsed_targets = (
            {"vim--gtk2": None, "png": None, "ruby%2.3": None},
            "repository",
        )
        cmd_out = {
            "retcode": 0,
            "stdout": "quirks-2.241 signed on 2016-07-26T16:56:10Z",
            "stderr": "",
        }
        run_all_mock = MagicMock(return_value=cmd_out)
        patches = {
            "cmd.run_all": run_all_mock,
            "pkg_resource.parse_targets": MagicMock(return_value=parsed_targets),
            "pkg_resource.stringify": MagicMock(),
            "pkg_resource.sort_pkglist": MagicMock(),
        }

        with patch.dict(openbsdpkg.__salt__, patches):
            with patch("salt.modules.openbsdpkg.list_pkgs", ListPackages()):
                added = openbsdpkg.install()
                expected = {
                    "png": {"new": "1.6.23", "old": ""},
                    "ruby": {"new": "2.3.1p1", "old": ""},
                }
                self.assertDictEqual(added, expected)
        expected_calls = [
            call("pkg_add -x -I png--%", output_loglevel="trace", python_shell=False),
            call(
                "pkg_add -x -I ruby--%2.3", output_loglevel="trace", python_shell=False
            ),
            call(
                "pkg_add -x -I vim--gtk2%", output_loglevel="trace", python_shell=False
            ),
        ]
        run_all_mock.assert_has_calls(expected_calls, any_order=True)
        self.assertEqual(run_all_mock.call_count, 3)

    def test_upgrade_available(self):
        """
        Test upgrade_available when an update is available.
        """
        ret = MagicMock(return_value="5.4.2p0")
        with patch("salt.modules.openbsdpkg.latest_version", ret):
            self.assertTrue(openbsdpkg.upgrade_available("zsh"))

    def test_upgrade_not_available(self):
        """
        Test upgrade_available when an update is not available.
        """
        ret = MagicMock(return_value="")
        with patch("salt.modules.openbsdpkg.latest_version", ret):
            self.assertFalse(openbsdpkg.upgrade_available("zsh"))

    def test_upgrade(self):
        """
        Test upgrading packages.
        """
        ret = {}
        pkg_add_u_stdout = [
            "quirks-2.402 signed on 2018-01-02T16:30:59Z",
            "Read shared items: ok",
        ]
        ret["stdout"] = "\n".join(pkg_add_u_stdout)
        ret["retcode"] = 0
        run_all_mock = MagicMock(return_value=ret)
        with patch.dict(openbsdpkg.__salt__, {"cmd.run_all": run_all_mock}):
            with patch("salt.modules.openbsdpkg.list_pkgs", ListPackages()):
                upgraded = openbsdpkg.upgrade()
                expected = {
                    "png": {"new": "1.6.23", "old": ""},
                    "ruby": {"new": "2.3.1p1", "old": ""},
                }
                self.assertDictEqual(upgraded, expected)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import pytest
from tests.support.case import ModuleCase
from tests.support.helpers import slowTest
from tests.support.runtests import RUNTIME_VARS


@pytest.mark.windows_whitelisted
class PillarModuleTest(ModuleCase):
    """
    Validate the pillar module
    """

    @slowTest
    def test_data(self):
        """
        pillar.data
        """
        grains = self.run_function("grains.items")
        pillar = self.run_function("pillar.data")
        self.assertEqual(pillar["os"], grains["os"])
        self.assertEqual(pillar["monty"], "python")
        if grains["os"] == "Fedora":
            self.assertEqual(pillar["class"], "redhat")
        else:
            self.assertEqual(pillar["class"], "other")

    @slowTest
    def test_issue_5449_report_actual_file_roots_in_pillar(self):
        """
        pillar['master']['file_roots'] is overwritten by the master
        in order to use the fileclient interface to read the pillar
        files. We should restore the actual file_roots when we send
        the pillar back to the minion.
        """
        self.assertIn(
            RUNTIME_VARS.TMP_STATE_TREE,
            self.run_function("pillar.data")["master"]["file_roots"]["base"],
        )

    @slowTest
    def test_ext_cmd_yaml(self):
        """
        pillar.data for ext_pillar cmd.yaml
        """
        self.assertEqual(self.run_function("pillar.data")["ext_spam"], "eggs")

    @slowTest
    def test_issue_5951_actual_file_roots_in_opts(self):
        self.assertIn(
            RUNTIME_VARS.TMP_STATE_TREE,
            self.run_function("pillar.data")["ext_pillar_opts"]["file_roots"]["base"],
        )

    @slowTest
    def test_pillar_items(self):
        """
        Test to ensure we get expected output
        from pillar.items
        """
        get_items = self.run_function("pillar.items")
        self.assertDictContainsSubset({"monty": "python"}, get_items)
        self.assertDictContainsSubset(
            {"knights": ["Lancelot", "Galahad", "Bedevere", "Robin"]}, get_items
        )

    @slowTest
    def test_pillar_command_line(self):
        """
        Test to ensure when using pillar override
        on command line works
        """
        # test when pillar is overwriting previous pillar
        overwrite = self.run_function("pillar.items", pillar={"monty": "overwrite"})
        self.assertDictContainsSubset({"monty": "overwrite"}, overwrite)

        # test when using additional pillar
        additional = self.run_function("pillar.items", pillar={"new": "additional"})

        self.assertDictContainsSubset({"new": "additional"}, additional)



# -*- coding: utf-8 -*-
"""
This is the default nodegroup matcher.
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

import salt.loader
import salt.utils.minions  # pylint: disable=3rd-party-module-not-gated

log = logging.getLogger(__name__)


def match(tgt, nodegroups=None, opts=None):
    """
    This is a compatibility matcher and is NOT called when using
    nodegroups for remote execution, but is called when the nodegroups
    matcher is used in states
    """
    if not opts:
        opts = __opts__
    if not nodegroups:
        log.debug("Nodegroup matcher called with no nodegroups.")
        return False
    if tgt in nodegroups:
        matchers = salt.loader.matchers(opts)
        return matchers["compound_match.match"](
            salt.utils.minions.nodegroup_comp(tgt, nodegroups)
        )
    return False



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.nftables as nftables

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class NftablesTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the nftables state
    """

    def setup_loader_modules(self):
        return {nftables: {}}

    def test_chain_present(self):
        """
            Test to verify the chain is exist.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(
            side_effect=[
                {"result": True, "comment": ""},
                {"result": False, "comment": ""},
                {"result": False, "comment": ""},
            ]
        )
        with patch.dict(nftables.__salt__, {"nftables.check_chain": mock}):
            ret.update(
                {
                    "comment": "nftables salt chain is already"
                    " exist in filter table for ipv4"
                }
            )
            self.assertDictEqual(nftables.chain_present("salt"), ret)

            mock = MagicMock(
                side_effect=[
                    {"result": True, "comment": ""},
                    {"result": False, "comment": ""},
                ]
            )
            with patch.dict(nftables.__salt__, {"nftables.new_chain": mock}):
                ret.update(
                    {
                        "changes": {"locale": "salt"},
                        "comment": "nftables salt chain in filter"
                        " table create success for ipv4",
                    }
                )
                self.assertDictEqual(nftables.chain_present("salt"), ret)

                ret.update(
                    {
                        "changes": {},
                        "comment": "Failed to create salt chain"
                        " in filter table:  for ipv4",
                        "result": False,
                    }
                )
                self.assertDictEqual(nftables.chain_present("salt"), ret)

    def test_chain_absent(self):
        """
            Test to verify the chain is absent.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(side_effect=[False, True])
        with patch.dict(nftables.__salt__, {"nftables.check_chain": mock}):
            ret.update(
                {
                    "comment": "nftables salt chain is already absent"
                    " in filter table for ipv4"
                }
            )
            self.assertDictEqual(nftables.chain_absent("salt"), ret)

            mock = MagicMock(return_value="")
            with patch.dict(nftables.__salt__, {"nftables.flush": mock}):
                ret.update(
                    {
                        "result": False,
                        "comment": "Failed to flush salt chain"
                        " in filter table:  for ipv4",
                    }
                )
                self.assertDictEqual(nftables.chain_absent("salt"), ret)

    def test_append(self):
        """
            Test to append a rule to a chain
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(return_value=[])
        with patch.object(nftables, "_STATE_INTERNAL_KEYWORDS", mock):
            mock = MagicMock(return_value={"result": True, "comment": "", "rule": "a"})
            with patch.dict(nftables.__salt__, {"nftables.build_rule": mock}):
                mock = MagicMock(
                    side_effect=[
                        {"result": True, "comment": ""},
                        {"result": False, "comment": ""},
                        {"result": False, "comment": ""},
                        {"result": False, "comment": ""},
                    ]
                )
                with patch.dict(nftables.__salt__, {"nftables.check": mock}):
                    ret.update(
                        {
                            "comment": "nftables rule for salt"
                            " already set (a) for ipv4"
                        }
                    )
                    self.assertDictEqual(
                        nftables.append("salt", table="", chain=""), ret
                    )

                    with patch.dict(nftables.__opts__, {"test": True}):
                        ret.update(
                            {
                                "result": None,
                                "comment": "nftables rule for salt needs"
                                " to be set (a) for ipv4",
                            }
                        )
                        self.assertDictEqual(
                            nftables.append("salt", table="", chain=""), ret
                        )

                    with patch.dict(nftables.__opts__, {"test": False}):
                        mock = MagicMock(
                            side_effect=[
                                {"result": True, "comment": ""},
                                {"result": False, "comment": ""},
                            ]
                        )
                        with patch.dict(nftables.__salt__, {"nftables.append": mock}):
                            ret.update(
                                {
                                    "changes": {"locale": "salt"},
                                    "comment": "Set nftables rule for salt"
                                    " to: a for ipv4",
                                    "result": True,
                                }
                            )
                            self.assertDictEqual(
                                nftables.append("salt", table="", chain=""), ret
                            )

                            ret.update(
                                {
                                    "changes": {},
                                    "comment": "Failed to set nftables"
                                    " rule for salt.\nAttempted rule was"
                                    " a for ipv4.\n",
                                    "result": False,
                                }
                            )
                            self.assertDictEqual(
                                nftables.append("salt", table="", chain=""), ret
                            )

    def test_insert(self):
        """
            Test to insert a rule into a chain
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(return_value=[])
        with patch.object(nftables, "_STATE_INTERNAL_KEYWORDS", mock):
            mock = MagicMock(return_value={"result": True, "comment": "", "rule": "a"})
            with patch.dict(nftables.__salt__, {"nftables.build_rule": mock}):
                mock = MagicMock(
                    side_effect=[
                        {"result": True, "comment": ""},
                        {"result": False, "comment": ""},
                        {"result": False, "comment": ""},
                        {"result": False, "comment": ""},
                    ]
                )
                with patch.dict(nftables.__salt__, {"nftables.check": mock}):
                    ret.update(
                        {
                            "comment": "nftables rule for salt already"
                            " set for ipv4 (a)"
                        }
                    )
                    self.assertDictEqual(
                        nftables.insert("salt", table="", chain=""), ret
                    )

                    with patch.dict(nftables.__opts__, {"test": True}):
                        ret.update(
                            {
                                "result": None,
                                "comment": "nftables rule for salt"
                                " needs to be set for ipv4 (a)",
                            }
                        )
                        self.assertDictEqual(
                            nftables.insert("salt", table="", chain=""), ret
                        )

                    with patch.dict(nftables.__opts__, {"test": False}):
                        mock = MagicMock(
                            side_effect=[
                                {"result": True, "comment": ""},
                                {"result": False, "comment": ""},
                            ]
                        )
                        with patch.dict(nftables.__salt__, {"nftables.insert": mock}):
                            ret.update(
                                {
                                    "changes": {"locale": "salt"},
                                    "comment": "Set nftables rule for"
                                    " salt to: a for ipv4",
                                    "result": True,
                                }
                            )
                            self.assertDictEqual(
                                nftables.insert(
                                    "salt", table="", chain="", position=""
                                ),
                                ret,
                            )

                            ret.update(
                                {
                                    "changes": {},
                                    "comment": "Failed to set nftables"
                                    " rule for salt.\nAttempted rule was"
                                    " a",
                                    "result": False,
                                }
                            )
                            self.assertDictEqual(
                                nftables.insert(
                                    "salt", table="", chain="", position=""
                                ),
                                ret,
                            )

    def test_delete(self):
        """
            Test to delete a rule to a chain
        """
        ret = {"name": "salt", "changes": {}, "result": None, "comment": ""}

        mock = MagicMock(return_value=[])
        with patch.object(nftables, "_STATE_INTERNAL_KEYWORDS", mock):
            mock = MagicMock(return_value={"result": True, "comment": "", "rule": "a"})
            with patch.dict(nftables.__salt__, {"nftables.build_rule": mock}):
                mock = MagicMock(
                    side_effect=[
                        {"result": False, "comment": ""},
                        {"result": True, "comment": ""},
                        {"result": True, "comment": ""},
                        {"result": True, "comment": ""},
                    ]
                )
                with patch.dict(nftables.__salt__, {"nftables.check": mock}):
                    ret.update(
                        {
                            "comment": "nftables rule for salt"
                            " already absent for ipv4 (a)",
                            "result": True,
                        }
                    )
                    self.assertDictEqual(
                        nftables.delete("salt", table="", chain=""), ret
                    )

                    with patch.dict(nftables.__opts__, {"test": True}):
                        ret.update(
                            {
                                "result": None,
                                "comment": "nftables rule for salt needs"
                                " to be deleted for ipv4 (a)",
                            }
                        )
                        self.assertDictEqual(
                            nftables.delete("salt", table="", chain=""), ret
                        )

                    with patch.dict(nftables.__opts__, {"test": False}):
                        mock = MagicMock(
                            side_effect=[
                                {"result": True, "comment": ""},
                                {"result": False, "comment": ""},
                            ]
                        )
                        with patch.dict(nftables.__salt__, {"nftables.delete": mock}):
                            ret.update(
                                {
                                    "result": True,
                                    "changes": {"locale": "salt"},
                                    "comment": "Delete nftables rule" " for salt a",
                                }
                            )
                            self.assertDictEqual(
                                nftables.delete(
                                    "salt", table="", chain="", position=""
                                ),
                                ret,
                            )

                            ret.update(
                                {
                                    "result": False,
                                    "changes": {},
                                    "comment": "Failed to delete nftables"
                                    " rule for salt.\nAttempted rule was a",
                                }
                            )
                            self.assertDictEqual(
                                nftables.delete(
                                    "salt", table="", chain="", position=""
                                ),
                                ret,
                            )

    def test_flush(self):
        """
            Test to flush current nftables state
        """
        ret = {"name": "salt", "changes": {}, "result": None, "comment": ""}
        mock = MagicMock(return_value=[])
        with patch.object(nftables, "_STATE_INTERNAL_KEYWORDS", mock):
            mock = MagicMock(
                side_effect=[
                    {"result": False, "comment": ""},
                    {"result": True, "comment": ""},
                    {"result": True, "comment": ""},
                    {"result": True, "comment": ""},
                ]
            )
            with patch.dict(nftables.__salt__, {"nftables.check_table": mock}):
                ret.update(
                    {
                        "comment": "Failed to flush table  in family"
                        " ipv4, table does not exist.",
                        "result": False,
                    }
                )
                self.assertDictEqual(nftables.flush("salt", table="", chain=""), ret)

                mock = MagicMock(
                    side_effect=[
                        {"result": False, "comment": ""},
                        {"result": True, "comment": ""},
                        {"result": True, "comment": ""},
                    ]
                )
                with patch.dict(nftables.__salt__, {"nftables.check_chain": mock}):
                    ret.update(
                        {
                            "comment": "Failed to flush chain  in table"
                            "  in family ipv4, chain does not exist."
                        }
                    )
                    self.assertDictEqual(
                        nftables.flush("salt", table="", chain=""), ret
                    )

                    mock = MagicMock(
                        side_effect=[
                            {"result": True, "comment": ""},
                            {"result": False, "comment": ""},
                        ]
                    )
                    with patch.dict(nftables.__salt__, {"nftables.flush": mock}):
                        ret.update(
                            {
                                "changes": {"locale": "salt"},
                                "comment": "Flush nftables rules in  table"
                                "  chain ipv4 family",
                                "result": True,
                            }
                        )
                        self.assertDictEqual(
                            nftables.flush("salt", table="", chain=""), ret
                        )

                        ret.update(
                            {
                                "changes": {},
                                "comment": "Failed to flush" " nftables rules",
                                "result": False,
                            }
                        )
                        self.assertDictEqual(
                            nftables.flush("salt", table="", chain=""), ret
                        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Gareth J. Greenaway <gareth@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.rsync as rsync

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class RsyncTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the Rsync state
    """

    def setup_loader_modules(self):
        return {rsync: {}}

    def test_syncronized_failed(self):
        """
        Test to perform an rsync.synchorized call that fails
        """
        ret = {"pid": 100, "retcode": 23, "stderr": "", "stdout": ""}
        _expected = {
            "changes": {},
            "comment": "Destination directory name was not found.",
            "name": "name",
            "result": False,
        }

        with patch.dict(rsync.__opts__, {"test": True}):
            mock = MagicMock(return_value=ret)
            with patch.dict(rsync.__salt__, {"rsync.rsync": mock}):
                self.assertDictEqual(rsync.synchronized("name", "source"), _expected)

        # Run again mocking os.path.exists as True
        ret = {
            "pid": 100,
            "retcode": 23,
            "stderr": "Something went wrong",
            "stdout": "",
        }
        _expected = {
            "changes": {},
            "comment": "Something went wrong",
            "name": "name",
            "result": False,
        }

        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch.dict(rsync.__opts__, {"test": False}):
                mock = MagicMock(return_value=ret)
                with patch.dict(rsync.__salt__, {"rsync.rsync": mock}):
                    self.assertDictEqual(
                        rsync.synchronized("name", "source"), _expected
                    )

    def test_syncronized(self):
        """
        Test to perform an rsync.synchorized call that succeeds
        """
        ret = {
            "pid": 100,
            "retcode": 0,
            "stderr": "",
            "stdout": (
                "sending incremental file list\nsnapshot.jar\n\n"
                "sent 106 bytes  received 35 bytes  282.00 bytes/sec\n"
                "total size is 0  speedup is 0.00"
            ),
        }
        _expected = {
            "changes": {"copied": "snapshot.jar", "deleted": "N/A"},
            "comment": (
                "- sent 106 bytes\n- received 35 bytes\n- "
                "282.00 bytes/sec\n- total size is 0\n- speedup is 0.00"
            ),
            "name": "name",
            "result": True,
        }

        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch.dict(rsync.__opts__, {"test": False}):
                mock = MagicMock(return_value=ret)
                with patch.dict(rsync.__salt__, {"rsync.rsync": mock}):
                    self.assertDictEqual(
                        rsync.synchronized("name", "source"), _expected
                    )

        # Second pass simulating the file being in place
        ret = {
            "pid": 100,
            "retcode": 0,
            "stderr": "",
            "stdout": (
                "sending incremental file list\n\n"
                "sent 106 bytes  received 35 bytes  "
                "282.00 bytes/sec\ntotal size is 0  "
                "speedup is 0.00"
            ),
        }
        _expected = {
            "changes": {},
            "comment": (
                "- sent 106 bytes\n- received "
                "35 bytes\n- 282.00 bytes/sec\n- total "
                "size is 0\n- speedup is 0.00"
            ),
            "name": "name",
            "result": True,
        }

        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch.dict(rsync.__opts__, {"test": False}):
                mock = MagicMock(return_value=ret)
                with patch.dict(rsync.__salt__, {"rsync.rsync": mock}):
                    self.assertDictEqual(
                        rsync.synchronized("name", "source"), _expected
                    )

    def test_syncronized_test_true(self):
        """
        Test to perform an rsync.synchorized call that fails
        """
        ret = {
            "pid": 100,
            "retcode": 23,
            "stderr": "Something went wrong",
            "stdout": "",
        }
        _expected = {"changes": {}, "comment": "- ", "name": "name", "result": None}

        with patch("os.path.exists", MagicMock(return_value=True)):
            with patch.dict(rsync.__opts__, {"test": True}):
                mock = MagicMock(return_value=ret)
                with patch.dict(rsync.__salt__, {"rsync.rsync": mock}):
                    self.assertDictEqual(
                        rsync.synchronized("name", "source"), _expected
                    )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import

# Import Salt Libs
import salt.modules.rpm_lowpkg as rpm

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


def _called_with_root(mock):
    cmd = " ".join(mock.call_args[0][0])
    return cmd.startswith("rpm --root /")


class RpmTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.modules.rpm
    """

    def setup_loader_modules(self):
        return {rpm: {"rpm": MagicMock(return_value=MagicMock)}}

    # 'list_pkgs' function tests: 2

    def test_list_pkgs(self):
        """
        Test if it list the packages currently installed in a dict
        """
        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(rpm.list_pkgs(), {})
            self.assertFalse(_called_with_root(mock))

    def test_list_pkgs_root(self):
        """
        Test if it list the packages currently installed in a dict,
        called with root parameter
        """
        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            rpm.list_pkgs(root="/")
            self.assertTrue(_called_with_root(mock))

    # 'verify' function tests: 2

    def test_verify(self):
        """
        Test if it runs an rpm -Va on a system, and returns the
        results in a dict
        """
        mock = MagicMock(
            return_value={"stdout": "", "stderr": "", "retcode": 0, "pid": 12345}
        )
        with patch.dict(rpm.__salt__, {"cmd.run_all": mock}):
            self.assertDictEqual(rpm.verify("httpd"), {})
            self.assertFalse(_called_with_root(mock))

    def test_verify_root(self):
        """
        Test if it runs an rpm -Va on a system, and returns the
        results in a dict, called with root parameter
        """
        mock = MagicMock(
            return_value={"stdout": "", "stderr": "", "retcode": 0, "pid": 12345}
        )
        with patch.dict(rpm.__salt__, {"cmd.run_all": mock}):
            rpm.verify("httpd", root="/")
            self.assertTrue(_called_with_root(mock))

    # 'file_list' function tests: 2

    def test_file_list(self):
        """
        Test if it list the files that belong to a package.
        """
        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(rpm.file_list("httpd"), {"errors": [], "files": []})
            self.assertFalse(_called_with_root(mock))

    def test_file_list_root(self):
        """
        Test if it list the files that belong to a package, using the
        root parameter.
        """

        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            rpm.file_list("httpd", root="/")
            self.assertTrue(_called_with_root(mock))

    # 'file_dict' function tests: 2

    def test_file_dict(self):
        """
        Test if it list the files that belong to a package
        """
        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            self.assertDictEqual(rpm.file_dict("httpd"), {"errors": [], "packages": {}})
            self.assertFalse(_called_with_root(mock))

    def test_file_dict_root(self):
        """
        Test if it list the files that belong to a package
        """
        mock = MagicMock(return_value="")
        with patch.dict(rpm.__salt__, {"cmd.run": mock}):
            rpm.file_dict("httpd", root="/")
            self.assertTrue(_called_with_root(mock))

    # 'owner' function tests: 1

    def test_owner(self):
        """
        Test if it return the name of the package that owns the file.
        """
        self.assertEqual(rpm.owner(), "")

        ret = "file /usr/bin/salt-jenkins-build is not owned by any package"
        mock = MagicMock(return_value=ret)
        with patch.dict(rpm.__salt__, {"cmd.run_stdout": mock}):
            self.assertEqual(rpm.owner("/usr/bin/salt-jenkins-build"), "")
            self.assertFalse(_called_with_root(mock))

        ret = {
            "/usr/bin/vim": "vim-enhanced-7.4.160-1.e17.x86_64",
            "/usr/bin/python": "python-2.7.5-16.e17.x86_64",
        }
        mock = MagicMock(
            side_effect=[
                "python-2.7.5-16.e17.x86_64",
                "vim-enhanced-7.4.160-1.e17.x86_64",
            ]
        )
        with patch.dict(rpm.__salt__, {"cmd.run_stdout": mock}):
            self.assertDictEqual(rpm.owner("/usr/bin/python", "/usr/bin/vim"), ret)
            self.assertFalse(_called_with_root(mock))

    def test_owner_root(self):
        """
        Test if it return the name of the package that owns the file,
        using the parameter root.
        """
        self.assertEqual(rpm.owner(), "")

        ret = "file /usr/bin/salt-jenkins-build is not owned by any package"
        mock = MagicMock(return_value=ret)
        with patch.dict(rpm.__salt__, {"cmd.run_stdout": mock}):
            rpm.owner("/usr/bin/salt-jenkins-build", root="/")
            self.assertTrue(_called_with_root(mock))

    # 'checksum' function tests: 2

    def test_checksum(self):
        """
        Test if checksum validate as expected
        """
        ret = {
            "file1.rpm": True,
            "file2.rpm": False,
            "file3.rpm": False,
        }

        mock = MagicMock(side_effect=[True, 0, True, 1, False, 0])
        with patch.dict(rpm.__salt__, {"file.file_exists": mock, "cmd.retcode": mock}):
            self.assertDictEqual(
                rpm.checksum("file1.rpm", "file2.rpm", "file3.rpm"), ret
            )
            self.assertFalse(_called_with_root(mock))

    def test_checksum_root(self):
        """
        Test if checksum validate as expected, using the parameter
        root
        """
        mock = MagicMock(side_effect=[True, 0])
        with patch.dict(rpm.__salt__, {"file.file_exists": mock, "cmd.retcode": mock}):
            rpm.checksum("file1.rpm", root="/")
            self.assertTrue(_called_with_root(mock))

    @patch("salt.modules.rpm_lowpkg.HAS_RPM", True)
    @patch("salt.modules.rpm_lowpkg.rpm.labelCompare", return_value=-1)
    @patch("salt.modules.rpm_lowpkg.log")
    def test_version_cmp_rpm(self, mock_log, mock_labelCompare):
        """
        Test package version if RPM-Python is installed

        :return:
        """
        self.assertEqual(-1, rpm.version_cmp("1", "2"))
        self.assertEqual(mock_log.warning.called, False)
        self.assertEqual(mock_labelCompare.called, True)

    @patch("salt.modules.rpm_lowpkg.HAS_RPM", False)
    @patch("salt.modules.rpm_lowpkg.HAS_RPMUTILS", True)
    @patch("salt.modules.rpm_lowpkg.rpmUtils", create=True)
    @patch("salt.modules.rpm_lowpkg.log")
    def test_version_cmp_rpmutils(self, mock_log, mock_rpmUtils):
        """
        Test package version if rpmUtils.miscutils called

        :return:
        """
        mock_rpmUtils.miscutils = MagicMock()
        mock_rpmUtils.miscutils.compareEVR = MagicMock(return_value=-1)
        self.assertEqual(-1, rpm.version_cmp("1", "2"))
        self.assertEqual(mock_log.warning.called, True)
        self.assertEqual(mock_rpmUtils.miscutils.compareEVR.called, True)
        self.assertEqual(
            mock_log.warning.mock_calls[0][1][0],
            "Please install a package that provides rpm.labelCompare for more accurate version comparisons.",
        )

    @patch("salt.modules.rpm_lowpkg.HAS_RPM", False)
    @patch("salt.modules.rpm_lowpkg.HAS_RPMUTILS", False)
    @patch("salt.utils.path.which", return_value=True)
    @patch("salt.modules.rpm_lowpkg.log")
    def test_version_cmp_rpmdev_vercmp(self, mock_log, mock_which):
        """
        Test package version if rpmdev-vercmp is installed

        :return:
        """
        mock__salt__ = MagicMock(return_value={"retcode": 12})
        with patch.dict(rpm.__salt__, {"cmd.run_all": mock__salt__}):
            self.assertEqual(-1, rpm.version_cmp("1", "2"))
            self.assertEqual(mock__salt__.called, True)
            self.assertEqual(mock_log.warning.called, True)
            self.assertEqual(
                mock_log.warning.mock_calls[0][1][0],
                "Please install a package that provides rpm.labelCompare for more accurate version comparisons.",
            )
            self.assertEqual(
                mock_log.warning.mock_calls[1][1][0],
                "Installing the rpmdevtools package may surface dev tools in production.",
            )

    @patch("salt.modules.rpm_lowpkg.HAS_RPM", False)
    @patch("salt.modules.rpm_lowpkg.HAS_RPMUTILS", False)
    @patch("salt.utils.versions.version_cmp", return_value=-1)
    @patch("salt.utils.path.which", return_value=False)
    @patch("salt.modules.rpm_lowpkg.log")
    def test_version_cmp_python(self, mock_log, mock_which, mock_version_cmp):
        """
        Test package version if falling back to python

        :return:
        """
        self.assertEqual(-1, rpm.version_cmp("1", "2"))
        self.assertEqual(mock_version_cmp.called, True)
        self.assertEqual(mock_log.warning.called, True)
        self.assertEqual(
            mock_log.warning.mock_calls[0][1][0],
            "Please install a package that provides rpm.labelCompare for more accurate version comparisons.",
        )
        self.assertEqual(
            mock_log.warning.mock_calls[1][1][0],
            "Falling back on salt.utils.versions.version_cmp() for version comparisons",
        )



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging

from tests.unit.utils.scheduler.base import SchedulerTestsBase

log = logging.getLogger(__name__)


class SchedulerHelpersTest(SchedulerTestsBase):
    """
    Test scheduler helper functions
    """

    def setUp(self):
        super(SchedulerHelpersTest, self).setUp()
        self.schedule.opts["loop_interval"] = 1

    def test_get_schedule(self):
        """
        verify that the _get_schedule function works
        when remove_hidden is True and schedule data
        contains enabled key
        """
        job_name = "test_get_schedule"
        job = {
            "schedule": {
                "enabled": True,
                job_name: {"function": "test.ping", "seconds": 60},
            }
        }
        # Add the job to the scheduler
        self.schedule.opts.update(job)

        ret = self.schedule._get_schedule(remove_hidden=True)
        self.assertEqual(job["schedule"], ret)



# -*- coding: utf-8 -*-

"""
Tests for the salt-run command
"""

from __future__ import absolute_import

import pytest
import salt.utils.files
import salt.utils.platform
import salt.utils.yaml
from tests.integration.utils import testprogram
from tests.support.case import ShellCase
from tests.support.helpers import skip_if_not_root, slowTest
from tests.support.mixins import ShellCaseCommonTestsMixin

USERA = "saltdev"
USERA_PWD = "saltdev"
HASHED_USERA_PWD = "$6$SALTsalt$ZZFD90fKFWq8AGmmX0L3uBtS9fXL62SrTk5zcnQ6EkD6zoiM3kB88G1Zvs0xm/gZ7WXJRs5nsTBybUvGSqZkT."


@pytest.mark.windows_whitelisted
class RunTest(ShellCase, testprogram.TestProgramCase, ShellCaseCommonTestsMixin):
    """
    Test the salt-run command
    """

    _call_binary_ = "salt-run"

    def _add_user(self):
        """
        helper method to add user
        """
        try:
            add_user = self.run_call("user.add {0} createhome=False".format(USERA))
            add_pwd = self.run_call(
                "shadow.set_password {0} '{1}'".format(
                    USERA,
                    USERA_PWD if salt.utils.platform.is_darwin() else HASHED_USERA_PWD,
                )
            )
            self.assertTrue(add_user)
            self.assertTrue(add_pwd)
            user_list = self.run_call("user.list_users")
            self.assertIn(USERA, str(user_list))
        except AssertionError:
            self.run_call("user.delete {0} remove=True".format(USERA))
            self.skipTest("Could not add user or password, skipping test")

    def _remove_user(self):
        """
        helper method to remove user
        """
        user_list = self.run_call("user.list_users")
        for user in user_list:
            if USERA in user:
                self.run_call("user.delete {0} remove=True".format(USERA))

    @slowTest
    def test_in_docs(self):
        """
        test the salt-run docs system
        """
        data = self.run_run("-d")
        data = "\n".join(data)
        self.assertIn("jobs.active:", data)
        self.assertIn("jobs.list_jobs:", data)
        self.assertIn("jobs.lookup_jid:", data)
        self.assertIn("manage.down:", data)
        self.assertIn("manage.up:", data)
        self.assertIn("network.wol:", data)
        self.assertIn("network.wollist:", data)

    @slowTest
    def test_notin_docs(self):
        """
        Verify that hidden methods are not in run docs
        """
        data = self.run_run("-d")
        data = "\n".join(data)
        self.assertNotIn("jobs.SaltException:", data)

    @slowTest
    def test_salt_documentation_too_many_arguments(self):
        """
        Test to see if passing additional arguments shows an error
        """
        data = self.run_run("-d virt.list foo", catch_stderr=True)
        self.assertIn(
            "You can only get documentation for one method at one time",
            "\n".join(data[1]),
        )

    @slowTest
    def test_exit_status_unknown_argument(self):
        """
        Ensure correct exit status when an unknown argument is passed to salt-run.
        """

        runner = testprogram.TestProgramSaltRun(
            name="run-unknown_argument", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        runner.setup()
        stdout, stderr, status = runner.run(
            args=["--unknown-argument"], catch_stderr=True, with_retcode=True,
        )
        self.assert_exit_status(
            status, "EX_USAGE", message="unknown argument", stdout=stdout, stderr=stderr
        )
        # runner.shutdown() should be unnecessary since the start-up should fail

    @slowTest
    def test_exit_status_correct_usage(self):
        """
        Ensure correct exit status when salt-run starts correctly.
        """

        runner = testprogram.TestProgramSaltRun(
            name="run-correct_usage", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        runner.setup()
        stdout, stderr, status = runner.run(catch_stderr=True, with_retcode=True,)
        self.assert_exit_status(
            status, "EX_OK", message="correct usage", stdout=stdout, stderr=stderr
        )

    @skip_if_not_root
    @slowTest
    def test_salt_run_with_eauth_all_args(self):
        """
        test salt-run with eauth
        tests all eauth args
        """
        args = ["--auth", "--eauth", "--external-auth", "-a"]
        self._add_user()
        for arg in args:
            run_cmd = self.run_run(
                "{0} pam --username {1} --password {2}\
                                   test.arg arg kwarg=kwarg1".format(
                    arg, USERA, USERA_PWD
                )
            )
            expect = [
                "args:",
                "    - arg",
                "kwargs:",
                "    ----------",
                "    kwarg:",
                "        kwarg1",
            ]
            self.assertEqual(expect, run_cmd)
        self._remove_user()

    @skip_if_not_root
    @slowTest
    def test_salt_run_with_eauth_bad_passwd(self):
        """
        test salt-run with eauth and bad password
        """
        self._add_user()
        run_cmd = self.run_run(
            "-a pam --username {0} --password wrongpassword\
                               test.arg arg kwarg=kwarg1".format(
                USERA
            )
        )
        expect = ['Authentication failure of type "eauth" occurred for user saltdev.']
        self.assertEqual(expect, run_cmd)
        self._remove_user()

    @slowTest
    def test_salt_run_with_wrong_eauth(self):
        """
        test salt-run with wrong eauth parameter
        """
        run_cmd = self.run_run(
            "-a wrongeauth --username {0} --password {1}\
                               test.arg arg kwarg=kwarg1".format(
                USERA, USERA_PWD
            )
        )
        expect = r"^The specified external authentication system \"wrongeauth\" is not available\tAvailable eauth types: auto, .*"
        self.assertRegex("\t".join(run_cmd), expect)
